/*  ProMoIJ_M3_Image_corrections is an ImageJ macro developed to correct bleaching and subtract background 
 *   using a filter of difference of Gaussians in images from two-photon life imaging experiments
    
    Copyright (C) 2017  Jorge Valero Gómez-Lobo, Iñaki Paris & Amanda Sierra.

  	ProMoIJ_M3_Image_corrections is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

     ProMoIJ_M3_Image_corrections is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//This macro has been developed by Iñaki Paris, Amanda Sierra & Jorge Valero (jorge.valero@achucarro.org). 
//If you have any doubt about how to use it, please contact me.


//License
Dialog.create("GNU GPL License");
Dialog.addMessage("ProMoIJ_M2_Registration (C) 2017 J. Valero, I. Paris & A. Sierra.");
Dialog.setInsets(10, 20, 0);
Dialog.addMessage("ProMoIJ_M3_Image_corrections comes with ABSOLUTELY NO WARRANTY");
Dialog.setInsets(0, 20, 0);
Dialog.addMessage("This is free software, and you are welcome to redistribute it under certain conditions. Click on the Help button for details.");
Dialog.addHelp("http://www.gnu.org/licenses/gpl.html");
Dialog.show();

var Optim="";
if (isOpen("Log")){
	selectWindow("Log");
	run("Close");
}
setBatchMode(true);


//Selects original image files to be processed. The processed images will be saved as originalname_Sub.tif
dir=getDirectory("Select the M3 input images folder: M2 Registered");
dirsubt=getDirectory("Select the M3 output images folder: M3 Corrected");

/*Inserts parameters for background substraction. A difference of Gaussians is used, using default parameters of minmum=1 and maximum=50. The optimal values of these parameters must be determined experimentally. 
It is recommended that the maximum value is larger than the maximum diameter of the structure to be analyzed.*/

images=getFileList(dir);
v=0;
while(endsWith(images[v], "/")) v++;
run("Bio-Formats Importer", "open=["+dir+images[v]+"] color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
getDimensions(width0, height0, channels0, slices0, frames0);
selectWindow(getTitle());
close();

/*The user provides the value of the following experimental and analysis parameters:
 * M3.1 Minimum sigma: Image smoothing (higher values = smoother image)
 * M3.2 Maximum sigma: Local background subtraction, (lower values = more subtraction)
 */
Dialog.create("ProMoIJ Macro 3 Image correction Parameters");

Dialog.addNumber("M3.1 Minimum sigma", 1);
Dialog.addNumber("M3.2 Maximum sigma", 50);
Dialog.addCheckbox("M3.3 Optimization mode", false);
Dialog.addHelp("http://www.achucarro.org/downloads");
Dialog.show();


gmin=Dialog.getNumber();
gmax=Dialog.getNumber();
ver=Dialog.getCheckbox();


fotos=getFileList(dir);
for (fot=0; fot<fotos.length; fot++){
	work();
	//Optimization paramaters dialog.
	if (ver==true){
	if (Optim=="Change parameters"){
			Dialog.create("ProMoIJ Macro 3 Image correction Parameters");
			
			Dialog.addNumber("M3.1 Minimum sigma", gmin);
			Dialog.addNumber("M3.2 Maximum sigma", gmax);
			Dialog.addCheckbox("M3.3 Optimization mode", true);
			Dialog.addHelp("http://www.achucarro.org/downloads");
			Dialog.show();
			
			gmin=Dialog.getNumber();
			gmax=Dialog.getNumber();
			ver=Dialog.getCheckbox();
			fot--;
		}
		if (Optim=="Correct all images with the current settings"){
			fot=-1;
			ver=false; 
		}
	}
}

	print("Images have been corrected!!!");
	for(i=0; i<300; i=i+20){
		beep();
		wait(i);
	}

//The function "work" opens the image files, corrects bleaching using the Histogram Matching ImageJ plugin, and performs Gaussian filtering. 
function work(){
	run("Bio-Formats Importer", "open=["+dir+fotos[fot]+"] autoscale color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT"); 
	name=getTitle();
	namenoext=File.nameWithoutExtension();
	getDimensions(width1, height1, channels1, slices1, frames1);
	firstslice=1;
	lastslice=slices1;
	firstframe=1;
	lastframe=frames1;
	run("Duplicate...", "title=temporal duplicate slices="+firstslice+"-"+lastslice+" frames="+firstframe+"-"+lastframe);
	selectWindow(name);
	close();
	selectWindow("temporal");
	run("Bleach Correction", "correction=[Histogram Matching]");
	selectWindow("DUP_temporal");
	difgaus(gmin,gmax, "DifGauss");
	selectWindow("DifGauss");
	save(dirsubt+namenoext+"_Sub.tif");
	run ("Close All");
	if (ver==true){
		setBatchMode(false);
		run("Close All");
		run("Bio-Formats Importer", "open=["+dir+fotos[fot]+"] autoscale color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
		rename("Original");
		run("Z Project...", "projection=[Max Intensity] all");
		rename("MAX_Original");
		run("Maximize");
		selectWindow("Original");
		close();
		run("Bio-Formats Importer", "open=["+dirsubt+namenoext+"_Sub.tif] autoscale color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
		rename("Corrected");
		run("Z Project...", "projection=[Max Intensity] all");
		rename("MAX_Corrected");
		run("Maximize");
		selectWindow("Corrected");
		close();
		run("Tile");
		waitForUser("Optimization", "Please, check the images and click OK.\n   \n                     Zoom in: Ctrl+\n                     Zoom out: Ctrl-");
		//Optimization dialog
		Dialog.create("Optimization mode");
		items=newArray("Check next image", "Change parameters", "Correct all images with the current settings");
		Dialog.addRadioButtonGroup("Select one option" , items, 3, 1, "Check next image");
		Dialog.addHelp("http://www.achucarro.org/downloads");
		Dialog.show;
		Optim=Dialog.getRadioButton();
		run ("Close All");
		setBatchMode(true);
	}
	
}

//The function "difgaus" performs the difference of Gaussians.
	
function difgaus (min, max, finalname){
	run("Duplicate...", "title=min duplicate");
	run("Gaussian Blur...", "sigma="+min+" stack");
	run("Duplicate...", "title=max duplicate");
	run("Gaussian Blur...", "sigma="+max+" stack");

	imageCalculator("Subtract stack", "min","max");
	selectWindow("max");
	close();
	selectWindow("min");
	rename(finalname);
}

