  /*  ProMoIJ_M5_Motility_analysis is an ImageJ macro developed to analyze 
  *  microglial process velocity, protraction, retraction and tip velocity.
    
    Copyright (C) 2017  Jorge Valero G mez-Lobo, I aki Paris & Amanda Sierra.

  	ProMoIJ_M5_Motility_analysis is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation either version 3 of the License, or
    (at your option) any later version

    ProMoIJ_M5_Motility_analysis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details

    You should have received a copy of the GNU General Public License
    along with this macro.  If not, see <http://www.gnu.org/licenses/>.
*/

//This macro has been developed by I aki Paris, Amanda Sierra & Jorge Valero (jorge.valero@achucarro.org). 
//If you have any doubt about its use, please contact me.

//General variables

var exclude=false;
var infovar=NaN;
var AverPre=20;
var perFiber=75;
var X0centralPost=0;
var Y0centralPost=0;
var Xincrement=0;
var Yincrement=0;
var X0central=0;
var Y0central=0;
var Xbounding=0;
var Ybounding=0;
var angle=0;
var gmin=0;
var step=0;
var thrsval=0;
var maxtolerance=0;
var ratio=0;
var maxdif=-0;
var rotang=0;
var roiFirst=false;
var stepper=0;
var range=0;
var smoothtype="Gaussian";
var Parameters="Continue";
var spacejump=0;
var xgapper=0.5;
var zgapper=0.5;
var absolutborder=false;
var nextcell=false;
var nextimage=false;


//License
Dialog.create("GNU GPL License");
Dialog.addMessage("ProMoIJ_M5_Motility_analysis (C) 2017 J. Valero, I. Paris & A. Sierra.");
Dialog.setInsets(10, 20, 0);
Dialog.addMessage("ProMoIJ_M5_Motility_analysis comes with ABSOLUTELY NO WARRANTY.");
Dialog.setInsets(0, 20, 0);
Dialog.addMessage("This is free software and you are welcome to redistribute it under certain conditions. Click on the Help button for details.");
Dialog.addHelp("http://www.gnu.org/licenses/gpl.html");
Dialog.show();


setBatchMode(true);

/*Selects the folders that contain the image files to be analyzed, the ROIs, and the folder to save the results table. 
 Four table files will be generated as .xls files:
 Process velocity: contains the process lenght in each time frame, and the resulting process velocity, protraction, and retraction between consecutive frames (microns per minute)
 Mean Process velocity: contains the mean process velocity values from each process over the time course of the experiment (microns per minute)
 Retraction: contains the mean retraction values from each process over the time course of the experiment (microns per minute)
 Protraction: contains the mean protraction values from each process over the time course of the experiment (microns per minute)
 Tip velocity: contains the coordinates (XYZ) of the tip position in each time frame, tip displacement, and tip velocity between consecutive frames (microns per minute)
 Mean tip velocity: contains total tip displacement and mean tip velocity over the time course of the experiment (microns per minute)
 */
dir=getDirectory("Please, select the M5 input images folder: M4 Processed");
dirRois=getDirectory("Please, select the M5 input ROIs folder: M4 Selection ROIs");
dirParam=getDirectory("Please, select the input/output parameters folder: M5 Parameters");
dirRes=getDirectory("Please, select the output results folder: M5 Results");


/*The user provides the value of the following experimental and analysis parameters in the "ProMoIJ Macro 5 Motility analysis Main Menu":
 * M5.1 Time interval ("timeinterval"): Time interval between frames, in minutes
 * M5.2 3D Gaussian sigma ("gmin"): 3D smoothing (higher values = smoother image)
 * M5.3 Skeleton smoothing range: Range (in pixels) for  3D skeleton smoothing, (higher values = smoother image)
 * M5.4 Maxima threshold: Intensity threshold used to identify a process
 * M5.5 Advanced menu: Activates the Advanced parameters menu
 * M5.6 Optimization mode: Activates the Optimization mode, which will generate images from each reconstructed process skeleton and allow the user to visualize them and then change parameter values
 */

//Pre-saved parameters are loaded, if they exists 
if(File.exists(dirParam+"Parameters.txt")){
	estring=File.openAsString(dirParam+"Parameters.txt");
	separau=split(estring," \n");
	Array.print(separau);
}

//otherwise default parameters are loaded
else separau=newArray(1.5, 1, 1, 100, 100, 0.25, 5, 2, 2, 0.5, 1,  "Gaussian", 2, "Base-to-tip", true, 20, 75, false);
if (separau[3]==separau[4]) ThrsTol=true;
else ThrsTol=false;

Dialog.create("ProMoIJ Macro 5 Motility analysis Main Menu");

Dialog.addNumber("M5.1 Time interval", separau[0], 3, 5, "minutes");
Dialog.setInsets(15, 20, 0);
Dialog.addNumber("M5.2 3D Gaussian sigma", separau[1]);
Dialog.setInsets(15, 20, 0);
Dialog.addNumber("M5.3 Skeleton smoothing range", separau[12], 0, 0, "pixels");
Dialog.setInsets(15, 20, 0);
Dialog.addNumber("M5.4 Maxima threshold", separau[3]);
Dialog.setInsets(-10, 00, 5);
Dialog.addMessage("_______________________________");

Dialog.addCheckbox("M5.5 Advanced Menu", false);
Dialog.addCheckbox("M5.6 Optimization mode", false);

Dialog.addHelp("http://www.achucarro.org/downloads");

Dialog.show();

timeinterval=Dialog.getNumber();
gmin=Dialog.getNumber();
range=Dialog.getNumber();
thrsval=Dialog.getNumber();

Admen=Dialog.getCheckbox();
visual=Dialog.getCheckbox();

if(ThrsTol==true)separau[4]=thrsval;


/* If "Advanced Menu" option was checked in the Main menu the "ProMoIJ Macro 5 Advanced Parameters" menu will be shown 
 * M5.7 Orientation vector direction: Indicates the direction of the orientation vector (base-to-tip or tip-to-base), along which the process will be aligned. The base of the process should be positioned close to the cell soma.
 * M5.8 Y step:  Distance in pixels between consecutive profile lines. A value of 1 will draw a profile line through every Y position of the re-oriented process.
 * M5.9 Tolerance: minimum difference between peaks intensities used to define a maxima 
 * M5.10 Relative values to find borders: Maximum intensity of the border is defined as certain ratio of the profile maxima 
 * M5.11 Border/maximum ratio: Ratio of profile maxima used to calculate maximum intensity of the border
 * M5.12 Border inflexion amplitude: Minimum amplitude of concave inflexions to separate close branching processes
 * M5.13 Activate pre-detection step: Pre-detection step used to refine process detection and avoid analysis of processes that do not fit into the ROI
 * M5.14 Percentage of ROI: Minimum percentage of the process selection ROI that should be reached by the process skeleton
 * M5.15 Process comparison range: Y range (in pixels) to search for the process in the previous time frame when branching is encountered
 * M5.16 Comparison height: Range of XZ coordinates of the previous time-frame to obtain the mean XZ coordinate that will be used to calculate distances when branching is encountered
 * M5.17 Size of gaps: Size (in pixels) of allowed process gaps
 * M5.18 X gap extension: Number of pixels used to expand X limits of previous XZ plane per Y position when a gap is encountered
 * M5.19 Z gap extension: Number of pixels used to expand Z limits of previous XZ plane per Y position when a gap is encountered
 * M5.20 Skeleton smoothing filter: type of skeleton smoothing filter, Gaussian or Mean
*/

if (Admen==true){
	

	item=newArray("Base-to-tip", "Tip-to-Base");
	item2=newArray("Gaussian", "Mean");
	Dialog.create("ProMoIJ Macro 5 Advanced Parameters");

	Dialog.setInsets(20, 20, 0);
	Dialog.addMessage("_______________________________");
	Dialog.setInsets(0, 20, -5);
	Dialog.addMessage("PROCESS DETECTION PARAMETERS");
	Dialog.setInsets(-10, 20, 5);
	Dialog.addMessage("_______________________________");
	Dialog.setInsets(-10, 20, 5);
	Dialog.addRadioButtonGroup("M5.7 Orientation vector direction", item, 1, 2, separau[13]); 
	Dialog.addNumber("M5.8 Y step", separau[2], 0, 2, "pixels");
	Dialog.addNumber("M5.9 Tolerance", separau[4]);
	Dialog.setInsets(0, 20, 5);
	Dialog.addCheckbox("M5.10 Relative values to find borders", separau[17]);
	Dialog.addNumber("M5.11 Border/maximum ratio", separau[5]);
	Dialog.addNumber("M5.12 Border inflexion amplitude", separau[6]);

	Dialog.setInsets(20, 20, 0);
	Dialog.addMessage("_______________________________");
	Dialog.setInsets(0, 20, -5);
	Dialog.addMessage("PROCESS SELECTION PARAMETERS");
	Dialog.setInsets(-10, 20, 5);
	Dialog.addMessage("_______________________________");
	Dialog.setInsets(-10, 20, 5);
	Dialog.addCheckbox("M5.13 Activate pre-detection step", separau[14]);
	Dialog.addNumber("M5.14 Percentage of ROI", separau[16], 0, 2, "%");
	Dialog.addNumber("M5.15 Process comparison range", separau[7], 0, 2, "pixels");
	Dialog.addNumber("M5.16 Comparison height", separau[15]);
	Dialog.addNumber("M5.17 Size of gaps", separau[8], 0, 2, "pixels");
	Dialog.addNumber("M5.18 X gap extension", separau[9], 2, 4, "pixels");
	Dialog.addNumber("M5.19 Z gap extension", separau[10], 2, 4, "pixels");
	Dialog.setInsets(-10, 20, 5);
	Dialog.addRadioButtonGroup("M5.20 Skeleton smoothing filter", item2, 1, 2, separau[11]);
	Dialog.addHelp("http://www.achucarro.org/downloads");
	Dialog.show();

	
	rotang=Dialog.getRadioButton();
	step=Dialog.getNumber();
	if (step<=0)  step=1;
	maxtolerance=Dialog.getNumber();
	absolutborder=Dialog.getCheckbox();
	ratio=Dialog.getNumber();
	maxdif=-Dialog.getNumber();
	roiFirst=Dialog.getCheckbox();
	perFiber=Dialog.getNumber();
	stepper=Dialog.getNumber();
	AverPre=Dialog.getNumber();
	jumpspace=Dialog.getNumber();
	xgapper=Dialog.getNumber();
	zgapper=Dialog.getNumber();
	smoothtype=Dialog.getRadioButton();
	if (maxtolerance!=thrsval) ThrsTol=false;
}
else {
	rotang=separau[13];
	step=parseFloat(separau[2]);
	if (step<=0)  step=1;
	maxtolerance=parseFloat(separau[4]);
	ratio=parseFloat(separau[5]);
	maxdif=-parseFloat(separau[6]);
	roiFirst=separau[14];
	perFiber=parseFloat(separau[16]);
	stepper=parseFloat(separau[7]);
	AverPre=parseFloat(separau[15]);
	jumpspace=parseFloat(separau[8]);
	xgapper=parseFloat(separau[9]);
	zgapper=parseFloat(separau[10]);
	smoothtype=separau[11];
	absolutborder=separau[17];
}



print("\nSelected parameters. \nM5.1 Time interval: "+timeinterval+"\nM5.2 3D Gaussian sigma: "+gmin+"\nM5.3 Skeleton smoothing range: "+range+"\nM5.4 Maxima threshold: "+thrsval+"\nM5.5 Advanced menu: "+Admen+"\nM5.6 Optimization mode: "+visual+"\nM5.7 Orientation vector direction: "+rotang+"\nM5.8 Y step: "+step+"\nM5.9 Tolerance: "+maxtolerance+"\nM5.10 Relative values to find borders: "+absolutborder+"\nM5.11 Border/maximum ratio: "+ratio+"\nM5.12 Border inflexion amplitude: "+maxdif+"\nM5.13 Activate pre-detection step: "+roiFirst+ "\nM5.14 Percentage of ROI: "+perFiber+"\nM5.15 Process comaparison range: " +stepper+"\nM5.16 Comparison height: "+AverPre+ "\nM5.17 Size of gaps: "+jumpspace+ "\nM5.18 X gap extension: "+xgapper+"\nM5.19 Z gap extension: "+zgapper+"\nM5.20 Skeleton smoothing filter: "+smoothtype);

tablearray=newArray("Image", "Cell", "Roi", "Time", "Length", "Net Process velocity (um/min)", "Absolute Process velocity");
tablecreator("Process velocity", tablearray);

tablearray=newArray("Image", "Cell", "Roi", "MotilityNet Process velocity (um/min)", "Absolute Process velocity");
tablecreator("Mean Process velocity", tablearray);

tablearray=newArray("Image", "Cell", "Roi", "Retraction (um/min)");
tablecreator("Retraction", tablearray);

tablearray=newArray("Image", "Cell", "Roi", "Protraction (um/min)");
tablecreator("Protraction", tablearray);

tablearray=newArray("Image", "Cell", "Roi", "Time point", "Xcrop", "Ycrop", "Zcrop", "Xreal", "Yreal", "Zreal", "Distance (um)", "Interval Tip velocity (um/min)");
tablecreator("Tip velocity", tablearray);

tablearray=newArray("Image", "Cell", "Roi", "Accumulated Distance(um)", "Total Tip velocity(um/min)", "Mean Distance per interval (um)", "Mean Tip velocity per interval(um/min)");
tablecreator("Mean Tip velocity", tablearray);


if (visual==true) setBatchMode(false);

fotos=getFileList(dir);
if (isOpen("ROI Manager")){ 
	selectWindow("ROI Manager");
	run("Close");
}

for (fot=0; fot<fotos.length; fot++){
	if (Parameters=="Analyze"){
		Parameters="Continue";
		//Pre-saved parameters are loaded
		estring=File.openAsString(dirParam+"Parameters.txt");
		separau=split(estring," \n");
		
		timeinterval=parseFloat(separau[0]);
		gmin=parseFloat(separau[1]);
		range=parseFloat(separau[12]);
		thrsval=parseFloat(separau[3]);
		Admen=false;
		visual=false;

		rotang=separau[13];
		step=parseFloat(separau[2]);
		if (step<=0)  step=1;
		maxtolerance=parseFloat(separau[4]);
		ratio=parseFloat(separau[5]);
		maxdif=-parseFloat(separau[6]);
		roiFirst=separau[14];
		perFiber=parseFloat(separau[16]);
		stepper=parseFloat(separau[7]);
		AverPre=parseFloat(separau[15]);
		jumpspace=parseFloat(separau[8]);
		xgapper=parseFloat(separau[9]);
		zgapper=parseFloat(separau[10]);
		smoothtype=separau[11];
		absolutborder=separau[17];		

		print("\nSelected parameters. \nM5.1 Time interval: "+timeinterval+"\nM5.2 3D Gaussian sigma: "+gmin+"\nM5.3 Skeleton smoothing range: "+range+"\nM5.4 Maxima threshold: "+thrsval+"\nM5.5 Advanced menu: "+Admen+"\nM5.6 Optimization mode: "+visual+"\nM5.7 Orientation vector direction: "+rotang+"\nM5.8 Y step: "+step+"\nM5.9 Tolerance: "+maxtolerance+"\nM5.10 Relative values to find borders: "+absolutborder+"\nM5.11 Border/maximum ratio: "+ratio+"\nM5.12 Border inflexion amplitude: "+maxdif+"\nM5.13 Activate pre-detection step: "+roiFirst+ "\nM5.14 Percentage of ROI: "+perFiber+"\nM5.15 Process comaparison range: " +stepper+"\nM5.16 Comparison height: "+AverPre+ "\nM5.17 Size of gaps: "+jumpspace+ "\nM5.18 X gap extension: "+xgapper+"\nM5.19 Z gap extension: "+zgapper+"\nM5.20 Skeleton smoothing filter: "+smoothtype);

		tablearray=newArray("Image", "Cell", "Roi", "Time", "Length", "Net Process velocity (um/min)", "Absolute Process velocity");
		tablecreator("Process velocity", tablearray);
		
		tablearray=newArray("Image", "Cell", "Roi", "MotilityNet Process velocity (um/min)", "Absolute Process velocity");
		tablecreator("Mean Process velocity", tablearray);
		
		tablearray=newArray("Image", "Cell", "Roi", "Retraction (um/min)");
		tablecreator("Retraction", tablearray);
		
		tablearray=newArray("Image", "Cell", "Roi", "Protraction (um/min)");
		tablecreator("Protraction", tablearray);
		
		tablearray=newArray("Image", "Cell", "Roi", "Time point", "Xcrop", "Ycrop", "Zcrop", "Xreal", "Yreal", "Zreal", "Distance (um)", "Interval Tip velocity (um/min)");
		tablecreator("Tip velocity", tablearray);
		
		tablearray=newArray("Image", "Cell", "Roi", "Accumulated Distance(um)", "Total Tip velocity(um/min)", "Mean Distance per interval (um)", "Mean Tip velocity per interval(um/min)");
		tablecreator("Mean Tip velocity", tablearray);
		setBatchMode(true);	
	}
	
	work();
	if(Parameters=="Analyze") fot=-1;
}

savetab("Process velocity", dirRes);
savetab("Mean Process velocity", dirRes);
savetab("Retraction", dirRes);
savetab("Protraction", dirRes);
savetab("Tip velocity", dirRes);
savetab("Mean Tip velocity", dirRes);
savetab("Log", dirRes);
saveparameters();
print("JOB DONE!!!");
for(i=0; i<300; i=i+20){
	beep();
	wait(i);
run("Close All");
}

//The function "work" will open the image files, their associated ROIs, and analyze the motility 
function work(){
	run("Colors...", "foreground=white background=black selection=yellow");
	run("Bio-Formats Importer", "open=["+dir+fotos[fot]+"] autoscale color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT"); 
	//Applies a Gaussian filter to smooth the contour using the selected 3D Gaussian sigma ("gmin")
	name=getTitle();
	namenoext=File.nameWithoutExtension();
	if (visual==true) setBatchMode(false);
	run("Duplicate...", "title=original duplicate");
	selectWindow(name);
	
	//Loop that opens the ROI files that correspond to the current image file
	roinames=getFileList(dirRois);
	for (r=0; r<roinames.length; r++){
		if (endsWith(roinames[r], namenoext+".zip")){
			roiManager("Open", dirRois+roinames[r]);
			/*Loop that selects the ROIs, locates them on the image, crops their bounding box, 
			 * clears the pixels outside the ROI, rotates the cropped image, and analyzes motility
			 */
			cuantos=roiManager("count");
			cuantos=cuantos/2;
			enga=0;
			for(buf=0; buf<cuantos; buf++){
				exclude=false;
				cropping("original", "crop", true, -1);
				//Activation of optimization mode
				if (visual==true) cropping("original", "originalcrop", false, -1);
				avoid=false;
				//Process pre-detection step
				if (roiFirst==true) Prefilamentdetector(0);
				else filamentdetector();
				//Detection and analysis step
				
				//Process exclusion
				if (exclude==true) {
					if (isOpen("crop")){
						selectWindow("crop");
						close();
					}
					print ("ROI "+ buf+ " of ROI file "+roinames[r]+" will be EXCLUDED of the analysis");
					//Optimization menu allows changing parameters or analyzing NEXT process, NEXT cell, or NEXT image
					if (visual==true){
						closerer();						
						Dialog.create("Optimization Menu");
						items = newArray("Change Parameters", "Continue");
						Dialog.addRadioButtonGroup("What would you like to do?" ,items, 2, 1, "Continue");
						Dialog.addCheckbox("Next Cell", false);
						Dialog.addCheckbox("Next Image", false);
						Dialog.addHelp("http://www.achucarro.org/downloads");
						Dialog.show();
						Parameters = Dialog.getRadioButton();
						nextcell=Dialog.getCheckbox();
						nextimage=Dialog.getCheckbox();
					}
					else Parameters="Continue";
				}
					if (Parameters=="Continue"){
							roiManager("Select", 0);
							roiManager("Delete");
							roiManager("Select", 0);
							roiManager("Delete");
							run("Select None");	
					}
					
					if (Parameters=="Change Parameters"){
							run("Select None");

							Dialog.create("ProMoIJ Macro 5 Motility analysis Main Menu");
							
							Dialog.addNumber("M5.1 Time interval", timeinterval, 3, 5, "minutes");
							Dialog.setInsets(15, 20, 0);
							Dialog.addNumber("M5.2 3D Gaussian sigma", gmin);
							Dialog.setInsets(15, 20, 0);
							Dialog.addNumber("M5.3 Skeleton smoothing range", range, 0, 0, "pixels");
							Dialog.setInsets(15, 20, 0);
							Dialog.addNumber("M5.4 Maxima threshold", thrsval);
							Dialog.setInsets(-10, 00, 5);
							Dialog.addMessage("_______________________________");
							
							Dialog.addCheckbox("M5.5 Advanced Menu", Admen);
							Dialog.addCheckbox("M5.6 Optimization mode", visual);

							Dialog.addHelp("http://www.achucarro.org/downloads");
							Dialog.show();
							
							timeinterval=Dialog.getNumber();
							gmin=Dialog.getNumber();
							range=Dialog.getNumber();
							thrsval=Dialog.getNumber();
							
							Admen=Dialog.getCheckbox();
							visual=Dialog.getCheckbox();

							if (ThrsTol==true) maxtolerance=thrsval;
							
							if (Admen==true){
								
								item=newArray("Base-to-tip", "Tip-to-Base");
								item2=newArray("Gaussian", "Mean");
								Dialog.create("ProMoIJ Macro 5 Advanced Parameters");
							
								Dialog.setInsets(20, 20, 0);
								Dialog.addMessage("_______________________________");
								Dialog.setInsets(0, 20, -5);
								Dialog.addMessage("PROCESS DETECTION PARAMETERS");
								Dialog.setInsets(-10, 20, 5);
								Dialog.addMessage("_______________________________");
								Dialog.setInsets(-10, 20, 5);
								Dialog.addRadioButtonGroup("M5.7 Orientation vector direction", item, 1, 2, rotang); 
								Dialog.addNumber("M5.8 Y step", step, 0, 2, "pixels");
								Dialog.addNumber("M5.9 Tolerance", maxtolerance);
								Dialog.setInsets(0, 20, 5);
								Dialog.addCheckbox("M5.10 Relative values to find borders", absolutborder);
								Dialog.addNumber("M5.11 Border/maximum ratio", ratio);
								Dialog.addNumber("M5.12 Border inflexion amplitude", -maxdif);
							
								Dialog.setInsets(20, 20, 0);
								Dialog.addMessage("_______________________________");
								Dialog.setInsets(0, 20, -5);
								Dialog.addMessage("PROCESS SELECTION PARAMETERS");
								Dialog.setInsets(-10, 20, 5);
								Dialog.addMessage("_______________________________");
								Dialog.setInsets(-10, 20, 5);
								Dialog.addCheckbox("M5.13 Activate pre-detection step", roiFirst);
								Dialog.addNumber("M5.14 Percentage of ROI", perFiber, 0, 2, "%");
								Dialog.addNumber("M5.15 Process comparison range", stepper, 0, 2, "pixels");
								Dialog.addNumber("M5.16 Comparison height", AverPre);
								Dialog.addNumber("M5.17 Size of gaps", jumpspace, 0, 2, "pixels");
								Dialog.addNumber("M5.18 X gap extension", xgapper, 2, 4, "pixels");
								Dialog.addNumber("M5.19 Z gap extension", zgapper, 2, 4, "pixels");
								Dialog.setInsets(-10, 20, 5);
								Dialog.addRadioButtonGroup("M5.20 Skeleton smoothing filter", item2, 1, 2, smoothtype);
								Dialog.addHelp("http://www.achucarro.org/downloads");
								Dialog.show();
							
								
								rotang=Dialog.getRadioButton();
								step=Dialog.getNumber();
								if (step<=0)  step=1;
								maxtolerance=Dialog.getNumber();
								absolutborder=Dialog.getCheckbox();
								ratio=Dialog.getNumber();
								maxdif=-Dialog.getNumber();
								roiFirst=Dialog.getCheckbox();
								perFiber=Dialog.getNumber();
								stepper=Dialog.getNumber();
								AverPre=Dialog.getNumber();
								jumpspace=Dialog.getNumber();
								xgapper=Dialog.getNumber();
								zgapper=Dialog.getNumber();
								smoothtype=Dialog.getRadioButton();
								if (maxtolerance!=thrsval) ThrsTol=false;
							}
							print("\nSelected parameters. \nM5.1 Time interval: "+timeinterval+"\nM5.2 3D Gaussian sigma: "+gmin+"\nM5.3 Skeleton smoothing range: "+range+"\nM5.4 Maxima threshold: "+thrsval+"\nM5.5 Advanced menu: "+Admen+"\nM5.6 Optimization mode: "+visual+"\nM5.7 Orientation vector direction: "+rotang+"\nM5.8 Y step: "+step+"\nM5.9 Tolerance: "+maxtolerance+"\nM5.10 Relative values to find borders: "+absolutborder+"\nM5.11 Border/maximum ratio: "+ratio+"\nM5.12 Border inflexion amplitude: "+maxdif+"\nM5.13 Activate pre-detection step: "+roiFirst+ "\nM5.14 Percentage of ROI: "+perFiber+"\nM5.15 Process comaparison range: " +stepper+"\nM5.16 Comparison height: "+AverPre+ "\nM5.17 Size of gaps: "+jumpspace+ "\nM5.18 X gap extension: "+xgapper+"\nM5.19 Z gap extension: "+zgapper+"\nM5.20 Skeleton smoothing filter: "+smoothtype);
							buf--;

					}
					if (nextcell==true || nextimage==true){
						buf=cuantos;
						if (nextcell==true) r++;
						if (nextimage==true) r=roinames.length;
						if (isOpen("ROI Manager")){ 
							selectWindow("ROI Manager");
							run("Close");
						}
					}
					if (Parameters=="Analyze"){
						buf=cuantos;
						r=roinames.length;
						if (isOpen("ROI Manager")){ 
							selectWindow("ROI Manager");
							run("Close");
						}
					}		
			}
		}
	}
	if (Parameters!="Analyze"){
		selectWindow(name);
		close();
		if(isOpen("original")){
			selectWindow("original");
		close();
		}
	}	
}

//The function "cropping" crops the PROCESS and rotates it using the function "reorient" along the orientation vector previously defined
function cropping(imagewin, imageout, optionclear, initial){
	if (visual==true) setBatchMode(false);
	selectWindow(imagewin);
	roiManager("Select", 0);
	getBoundingRect(Xbounding, Ybounding, wbound, hbound);
	run("Duplicate...", "title="+imageout+" duplicate");
	roiManager("Select", 1);
	reorient();
	run("Select None");
}

//The function "reorient" rotates the cropped image along the axis defined by the reorientation vector to make it vertical
function reorient(){
	getDimensions(width, height, channels, slices, frames);
	run("Measure");
	angle2=getResult("Angle", 0);
	if (initial>0) {
		if (rotang=="Base-to-tip") angle2=angle2-90;
		else angle2=angle2+90;
	}
	else{
		if (rotang=="Base-to-tip") angle2=angle2+90;
		else angle2=angle2-90;
		angle=angle2;
	}
	run("Select None");
	run("Rotate... ", "angle="+angle2+" grid=4 interpolation=Bilinear fill enlarge stack");
	roiManager("Select", 0);
	run("Rotate...", "  angle="+angle2);
	run("Crop");
	if (optionclear==true){
		run("Clear Outside", "stack"); 
		if (initial>0) frameprojector("Average", imageout);
		run("Gaussian Blur 3D...", "x="+gmin+" y="+gmin+" z="+gmin);
		roiManager("Select", 0);
	}
	else{
		setForegroundColor(0, 0, 0);
		run("Draw", "hyperstack");
		setForegroundColor(255, 255, 255);
	}
	getDimensions(widthPost, heightPost, channels, slices, frames);
	selectWindow("Results");
	run("Close");
	if (initial<0){
		X0central=(width/2);
		Y0central=(height/2);
		Xincrement=widthPost-width;
		Yincrement=heightPost-height;
		X0centralPost=(width/2)+(Xincrement/2);
		Y0centralPost=(height/2)+(Yincrement/2);	
	}
}

//The function "saveparameters" prints and saves parameters as a text file named "Parameteres.txt"
function saveparameters(){
	selectWindow("Log");
	run("Close");
	print (timeinterval);
	print (gmin);
	print (step);
	print (thrsval); 
	print (maxtolerance);
	print (ratio); 
	print (maxdif*(-1));
	print(stepper);
	print(jumpspace);
	print(xgapper);
	print(zgapper);
	print(smoothtype);
	print(range); 
	print (rotang); 
	print(roiFirst);
	print(AverPre);
	print(perFiber);
	print(absolutborder);
	selectWindow("Log");
	saveAs("Text",dirParam+"Parameters.txt");
	}

/* The function filamentdetector finds the center of each reoriented process through the Y axis. It first finds the peak intensity (maximum),
 *  of each profile line. Next, it defines the borders (in the X axis) of each process as positions with a certain intensity below peak intensity value.
 *  In case the process splits into two terminal branches, the border of each one will be defined as a minimum difference of intensity (5 as default value). 
 *  Next, this procedure is performed along the Z axis to determine de 2D coordinates of the process center in each Y position.  
 *  The 2D coordinates of consecutive Y positions are compared to ensure that the processes are physically connected.
 *  The lenght of each process is calculated as the sum of the distances between each consecutive (in the Y axis) pair of process centers
 *  Process velocity is finally calculated as the lenght of each process over the time increment between consecutive time frames.
 *  Tip velocity is calculated as the tip displacement over the time increment between consecutive time frames.
 */

function filamentdetector(){
	//Obtains the image dimensions  
	getDimensions(width, height, channels, slices, frames);
	contador2=0;
	sumMotility=0;
	sumAbsMotility=0;
	sumretrac=0;
	sumprotrac=0;
	retracounter=0;
	sumprotrac=0;
	protracounter=0;
	
	//This loop determines the process lenght in each consecutive time frame
	if (visual==true) concatenate="title=[Concatenated Stacks]";
	sumInterSpeed=NaN;
	sumDisTip=NaN;
	interVals=NaN;
	DisTip=NaN;
	interSpeed=NaN;
	for (f=1; f<=frames; f++){
		NOmaxima=0;
		YsPerFrame=0;
		if (f==1 && roiFirst==true){
			selectWindow("MeanZYZpoints");
			tableinfo=getInfo();
			Ltab=split(tableinfo, "\n");
			sizearr=Ltab.length;
			XpositPrev=newArray(sizearr);
			YpositPrev=newArray(sizearr);
			ZpositPrev=newArray(sizearr);
			for (i=1; i<Ltab.length; i++){
				infoTab("MeanZYZpoints", i, 0);
				YpositPrev[i-1]=parseFloat(infovar);
				infoTab("MeanZYZpoints", i, 1);
				XpositPrev[i-1]=parseFloat(infovar);
				infoTab("MeanZYZpoints", i, 2);
				ZpositPrev[i-1]=parseFloat(infovar);	
			}
			selectWindow("MeanZYZpoints");
			run("Close");
		}
		if (f>1){			
			XpositPrev=Array.copy(Xposit);
			YpositPrev=Array.copy(Yposit);
			ZpositPrev=Array.copy(Zposit);	
		}
		
		Xposit=newArray();
		Yposit=newArray();
		Zposit=newArray();
	
		if (visual==true) setBatchMode(false);
	 	selectWindow("crop");
	 	run("Select None");
	 	if (slices>1) run("Duplicate...", "title=frame duplicate frames="+f);
	 	else {
	 		setSlice(f);
	 		run("Duplicate...", "title=frame");
	 	}
	 	if (visual==true) {
	 		selectWindow("originalcrop");
	 		if (slices>1) run("Duplicate...", "title=test"+f+" duplicate frames="+f);
	 		else {
		 		setSlice(f);
		 		run("Duplicate...", "title=test"+f);
		 	}
	 	}
		selectWindow("frame");
		sumd=0;
		coorb=newArray();
		//This loop draws profile lines through the Y axis of the process at the selected Y steps ("step")
		for (i=0; i<=height; i=i+step){
			coor=newArray();
			coortemp=newArray(5);
			
			//This loop will draw the profile lines at each Y step through the Z axis
			if (visual==true) setBatchMode(true);
			for (stack=1; stack<=slices; stack++){
				fibra=0;
				selectWindow("frame");
				//Obtains the intensity data of each profile line and creates an array with the XYZ position of the intensity peaks
				// using the selected maximum intensity difference between peak and valley ("maxtolerance" variable)
				setSlice(stack);
				makeLine(0, i, width, i);
				prof=getProfile();
				max=Array.findMaxima(prof, maxtolerance);
				Array.sort(max);
				//This loop will determine the borders of the process at each Y position
				for (ii=0; ii<max.length; ii++){
					//Determines the X coordinates of the right border as a ratio ("ratio") of the maximum peak intensity ("minInt")
					//The position of the peak ("origin") is the starting point for searching the borders.
					counterR=0;
					counter2R=0;
					origin=max[ii];
					//Borders will be determined only if the maxima intensity is above the threshold ("thrsval")
					if (prof[origin]>=thrsval){
						dif=0;
						if (absolutborder==true) minInt=prof[origin]*ratio;
						else minInt=thrsval;
						if (origin+1<prof.length){
							//Loop to compare intensities through the profile line between them and with the intensity to define a border ("minInt")
							//In case the process splits into two terminal branches, the defined border inflexion amplitude
							//("maxdif") will be used to determine the border as the valley between two peaks
							out=0;
							do{
								counter2R++;
								dif=prof[origin+counterR]-prof[origin+counter2R];
								if (prof[origin+counter2R]<minInt) out=1;
								else if(prof[origin+counter2R]<prof[origin+counterR]) counterR=counter2R;
							}while ((origin+counter2R+1)<prof.length && dif>maxdif && out==0);
							if (dif>maxdif) counter2R=counterR;
						}
						
						//Determines the X coordinates of the left border as a ratio ("ratio") of the maximum peak intensity ("minInt")
						//The position of the peak ("origin") is the starting point for searching the borders.
						counterL=0;
						counter2L=0;
						dif=0;
						if (origin-1>=0){
							//Loop to compare intensities through the profile line between them and with the intensity to define a border ("minInt")
							//In case the process splits into two terminal branches, the defined border inflexion amplitude
							//("maxdif") will be used to determine the border as the valley between two peaks
							out=0;
							do{
								counter2L++;
								dif=prof[origin-counterL]-prof[origin-counter2L];
								if (prof[origin-counter2L]<minInt) out=1;
								else if (prof[origin-counter2L]<prof[origin-counterL]) counterL=counter2L;	
							} while ((origin-counter2L-1)>=0 && dif>maxdif && out==0);
							if (dif>maxdif) counter2L=counterL;
						}
						
						//X coordinates of the left ("positionL") and right ("positionR") borders are used to obtain the X coordinate of the center of the process ("position")
						//The (5) coordinates of the left border (X), center (X), right border (X) at each Y and Z coordinate are stored in the array "coor" 
						positionL=origin-counter2L;
						positionR=origin+counter2R;
						position=((positionR+positionL)/2);
						coortemp[0]=positionL;
						coortemp[1]=position;
						coortemp[2]=positionR;
						coortemp[3]=i;
						coortemp[4]=stack;
						coor=Array.concat(coor, coortemp);
						fibra++;
					}
				}
			}
			//If more than two borders are found at the same YZ location, it discriminates whether there is a single or more processes
			if(coor.length>5){
				counter=0;
				//Loop to merge connected processes (in X). Each group of 5 coordinates is compared with the others so that if an overlap between the line defined by the X coordinates at each Y and Z position, 
				//the two most distal X coordinates are taken as the borders of a single process.
				for (arele=0; arele<coor.length-5; arele=arele+5) {
					counter=arele+5;
					for(arele2=counter; arele2<coor.length; arele2=arele2+5){	
						Z=coor[arele+4];
						Z2=coor[arele2+4];
						if (Z==Z2){
							XL=coor[arele];
							XR=coor[arele+2];
							XL2=coor[arele2];
							XR2=coor[arele2+2];
							if(XL<=XR2 && XR>=XL2) {
								if (XL<XL2) coor[arele]=XL;
								else coor[arele]=XL2;
								if (XR>XR2) coor[arele+2]=XR;
								else coor[arele+2]=XR2;
								coor[arele+1]=(coor[arele]+coor[arele+2])/2;
								coorL=Array.slice(coor,0,arele2);
								coorR=Array.slice(coor, arele2+5, coor.length); 
								coor=Array.concat(coorL, coorR);
								arele2=arele2-5;
								counter=counter-5;
							}
						}
					}
				}
			}
			
			//Loop to obtain the distance between the process center at each defined Y position ("step"). 
						
			if(coorb.length>1 && coor.length>1){
				NOmaxima=(coor[3]-coorb[3]-step);
				if((coorb[3]==(coor[3]-step)-NOmaxima) && NOmaxima<=jumpspace){
					
					Zgapranger=NOmaxima*zgapper;
					Xgapranger=NOmaxima*xgapper;
					NOmaxima=0;
					PAIRS=newArray();
					origin=newArray();
					origintemp=newArray(2);

					//Loop to identify connected processes (in Y). Each group of X, Z coordinates is compared in consecutive Y positions so that if an overlap of the area defined by X,Z is found, the center of this area is determined (array "PAIRS").
					//The distance between the center of each pair of connected areas is calculated. Only the minimal distance between consecutive Y positions is stored to calculate the total length.
					//If a second terminal branch appears, its distance to the previous center will be larger and not taken into account. Thus, only the length of one terminal branch is calculated.
					for (w=0;w<coorb.length;w=w+5){
						for(ww=0;ww<coor.length;ww=ww+5){
							Z=coorb[w+4];
							Z2=coor[ww+4];
							if (Z>=(Z2-Zgapranger) && Z<=(Z2+Zgapranger)){
								XL=coorb[w];
								XR=coorb[w+2];
								XL2=coor[ww];
								XR2=coor[ww+2];
								if (XL<=(XR2+Xgapranger) && XR>=(XL2-Xgapranger)){
									origintemp[0]=ww;
									origintemp[1]=ww+5;
									
									//Loop to obtain the deep X, Z coordinates to define the area occupied by the process in each Y position
									RA=newArray();
									poscomp=w;
									for(arele=w+5; arele<coorb.length; arele=arele+5){
										Z=coorb[poscomp+4];
										Z2=coorb[arele+4];
										if (Z==Z2-1){
											XL=coorb[poscomp];
											XR=coorb[poscomp+2];
											XL2=coorb[arele];
											XR2=coorb[arele+2];
											if (XL<=XR2 && XR>=XL2){
												RAT=newArray(XL2, XR2, coorb[arele+3], coorb[arele+4]);
												RA=Array.concat(RA,RAT);
												poscomp=arele;
											}
										}
									}
											
									//Loop to obtain the surface X, Z coordinates to define the area occupied by the process in each Y position
									LA=newArray();
									poscomp=w;									
									for(arele=w-5; arele>=0; arele=arele-5){
										Z=coorb[poscomp+4];
										Z2=coorb[arele+4];
										if (Z==Z2+1){
											XL=coorb[poscomp];
											XR=coorb[poscomp+2];
											XL2=coorb[arele];
											XR2=coorb[arele+2];
											if (XL<=XR2 && XR>=XL2){
												LAT=newArray(XL2, XR2, coorb[arele+3], coorb[arele+4]);
												LA=Array.concat(LAT,LA);
												poscomp=arele;
											}
										}
									}
											
									//Creates an array ("finalY") that defines the X, Z area occupied by the process at each Y position
									MA=newArray(coorb[w], coorb[w+2], coorb[w+3], coorb[w+4]);
									finalY=Array.concat(LA,MA);
									finalY=Array.concat(finalY, RA);
																									
									//Calculates the center point of the process in the Z axis 
									contador=0;
									sumatorio=0;
									for (psp=3; psp<finalY.length; psp=psp+4){
										contador++;
										sumatorio=sumatorio+finalY[psp];
									}
									Zmed=sumatorio/contador;

									//Calculates the center point of the process in the X axis 							
									ZmedEnt=round(Zmed);
									POSYtemp=newArray(3);
									if (Zmed!=ZmedEnt){
										for (psp=3; psp<finalY.length; psp=psp+4){
											contador++;
											if (finalY[psp]==ZmedEnt){
														
												POSYtemp[0]=(finalY[psp-3]+finalY[psp-2]+finalY[psp-7]+finalY[psp-6])/4;
												POSYtemp[1]=finalY[psp-1];
												POSYtemp[2]=Zmed;
											}
										}
									}
									else {
										for (psp=3; psp<finalY.length; psp=psp+4){
											contador++;
											if (finalY[psp]==Zmed){
														
												POSYtemp[0]=(finalY[psp-3]+finalY[psp-2])/2;
												POSYtemp[1]=finalY[psp-1];
												POSYtemp[2]=Zmed;
											}
										} 
									}
									
									//The X, Z coordinates of the center of the process at the current Y position are stored in "PAIRS"
									PAIRS=Array.concat(PAIRS, POSYtemp);

									//The same procedure is repeated for the next Y position, as defined by "step"
									
									//Loop to obtain the deep X, Z coordinates to define the area occupied by the process in each Y position
									RA=newArray();
									poscomp=ww;
									for(arele=ww+5; arele<coor.length; arele=arele+5){
											
										Z=coor[poscomp+4];
										Z2=coor[arele+4];
				
										if (Z==Z2-1){
											XL=coor[poscomp];
											XR=coor[poscomp+2];
											XL2=coor[arele];
											XR2=coor[arele+2];
											if (XL<=XR2 && XR>=XL2){
												RAT=newArray(XL2, XR2, coor[arele+3], coor[arele+4]);
												RA=Array.concat(RA,RAT);
												origintemp[1]=arele+5;
												poscomp=arele;
											}
										}
									}
											
									//Loop to obtain the surface X, Z coordinates to define the area occupied by the process in each Y position
									LA=newArray();
									poscomp=ww;
										
									for(arele=ww-5; arele>=0; arele=arele-5){
											
										Z=coor[poscomp+4];
										Z2=coor[arele+4];
										if (Z==Z2+1){
											XL=coor[poscomp];
											XR=coor[poscomp+2];
											XL2=coor[arele];
											XR2=coor[arele+2];
											if (XL<=XR2 && XR>=XL2){
												LAT=newArray(XL2, XR2, coor[arele+3], coor[arele+4]);
												LA=Array.concat(LAT,LA);
												origintemp[0]=arele;
												poscomp=arele;
											}
										}
									}
									origin=Array.concat(origin, origintemp);
								
								    //Creates an array ("finalY") that defines the X, Z area occupied by the process at each Y position
									MA=newArray(coor[ww], coor[ww+2], coor[ww+3], coor[ww+4]);
									finalY=Array.concat(LA,MA);
									finalY=Array.concat(finalY, RA);
									
									//Calculates the center point of the process in the Z axis 
									contador=0;
									sumatorio=0;
									for (psp=3; psp<finalY.length; psp=psp+4){
										contador++;
										sumatorio=sumatorio+finalY[psp];
									}
									Zmed=sumatorio/contador;
									
									//Calculates the center point of the process in the X axis 	
									ZmedEnt=round(Zmed);
									POSY2temp=newArray(3);
									if (Zmed!=ZmedEnt){
										for (psp=3; psp<finalY.length; psp=psp+4){
											contador++;
											if (finalY[psp]==ZmedEnt){
														
												POSYtemp[0]=(finalY[psp-3]+finalY[psp-2]+finalY[psp-7]+finalY[psp-6])/4;
												POSYtemp[1]=finalY[psp-1];
												POSYtemp[2]=Zmed;
											}
										}
									}
									else {
										for (psp=3; psp<finalY.length; psp=psp+4){
											contador++;
											if (finalY[psp]==Zmed){
														
												POSYtemp[0]=(finalY[psp-3]+finalY[psp-2])/2;
												POSYtemp[1]=finalY[psp-1];
												POSYtemp[2]=Zmed;
											}
										} 
									}
									//The X, Z coordinates of the center of the process at the current Y position are stored in "PAIRS";
									PAIRS=Array.concat(PAIRS, POSYtemp);
								}
							}
						}
					}
					
		
					//Removes potential repeated pairs of coordinates of the process center
					origintemp=newArray(2);
					for (w=0; w<PAIRS.length-6; w=w+6){
						XA=PAIRS[w];
						YA=PAIRS[w+1];
						ZA=PAIRS[w+2];
						XAA=PAIRS[w+3];
						YAA=PAIRS[w+4];
						ZAA=PAIRS[w+5];
						for (ww=6; ww<PAIRS.length; ww=ww+6){
							XB=PAIRS[ww];
							YB=PAIRS[ww+1];
							ZB=PAIRS[ww+2];
							XBB=PAIRS[ww+3];
							YBB=PAIRS[ww+4];
							ZBB=PAIRS[ww+5];
							if (XA==XB && YA==YB && ZA==ZB && XAA==XBB && YAA==YBB && ZAA==ZBB && w!=ww){
								PAIRSL=Array.slice(PAIRS,0,ww);
								PAIRSR=Array.slice(PAIRS, ww+6, PAIRS.length); 
								PAIRS=Array.concat(PAIRSL, PAIRSR);
								origintemp[0]=origin[w/3];
								origintemp[1]=origin[(ww/3)+1];
								origin[w/3]=origintemp[0];
								origin[w/3+1]=origintemp[1];
								originL=Array.slice(origin,0,ww/3);
								originR=Array.slice(origin, (ww/3)+2, origin.length);
								origin=Array.concat(originL, originR);
								ww=ww-6;
							}
						}
					}
					
					if (PAIRS.length>0){
						//Transforms pixels into length (as defined by the image file parameters) and caldulates the distance between the centers of consecutive connected processes. Only the minimal distance is stored.
						//The total length is calculated as the sum of the mimimal distances
						YsPerFrame++;
						if (YsPerFrame==1){
								XpositTemp=newArray(2);
								YpositTemp=newArray(2);
								ZpositTemp=newArray(2);
							}
						else{ 
								XpositTemp=newArray(1);
								YpositTemp=newArray(1);
								ZpositTemp=newArray(1);
							}
							precompare=false;
						for (e=0; e<PAIRS.length; e=e+6){
							
							getVoxelSize(xs, ys, zs, unit);
							x1=PAIRS[e+3]*xs;
							x0=PAIRS[e]*xs;
							y1=PAIRS[e+4]*ys;
							y0=PAIRS[e+1]*ys;
							z1=PAIRS[e+5]*zs;
							z0=PAIRS[e+2]*zs;
							
							midx01=(x1+x0)/2;
							midy01=(y1+y0)/2;
							midz01=(z1+z0)/2;
							//Distances of the current pairs of points to the previous process in the same Ys
							
							y0pre=NaN;
							y1pre=NaN;
							x0pre=NaN;
							x1pre=NaN;
							z0pre=NaN;
							z1pre=NaN;
							recounter=0;
							
							if (f>1 || (avoid==true && f==1)){
								do {
									for (dd=0; dd<YpositPrev.length; dd++){
										if (YpositPrev[dd]==PAIRS[e+1]+recounter || YpositPrev[dd]==PAIRS[e+1]-recounter ){
											x0pre=XpositPrev[dd]*xs;
											z0pre=ZpositPrev[dd]*zs;
											y0pre=YpositPrev[dd]*ys;
											nnn=1;
											for (hh=dd+1; hh<YpositPrev.length && hh<dd+AverPre; hh++){
												x0pre=x0pre+XpositPrev[hh]*xs;
												z0pre=z0pre+ZpositPrev[hh]*zs;
												y0pre=y0pre+YpositPrev[hh]*ys;
												nnn++;
											}
											x0pre=x0pre/nnn;
											z0pre=z0pre/nnn;
											y0pre=y0pre/nnn;
										}
										if (YpositPrev.length>dd+1){
											if ((YpositPrev[dd+1]==PAIRS[e+4]+recounter) || (YpositPrev[dd+1]==PAIRS[e+4]-recounter)){
												x1pre=XpositPrev[dd+1]*xs;
												z1pre=ZpositPrev[dd+1]*zs;
												y1pre=YpositPrev[dd+1]*ys;
												nnn=1;
												for (hh=dd+2; hh<YpositPrev.length && hh<dd+AverPre; hh++){
													x1pre=x1pre+XpositPrev[hh]*xs;
													z1pre=z1pre+ZpositPrev[hh]*zs;
													y1pre=y1pre+YpositPrev[hh]*ys;
													nnn++;
												}
												x1pre=x1pre/nnn;
												z1pre=z1pre/nnn;
												y1pre=y1pre/nnn;
											}	
										}
									}
									recounter++; 
								}
								while((y0pre==NaN && y1pre==NaN) && recounter<=stepper);
							}
					
							if (y0pre!=NaN || y1pre!=NaN){
								if (y0pre!=NaN && y1pre!=NaN){
									midx01pre=(x1pre+x0pre)/2;
									midy01pre=(y1pre+y0pre)/2;
									midz01pre=(z1pre+z0pre)/2;
								}
								if (y0pre==NaN) {
									midx01pre=x1pre;
									midy01pre=y1pre;
									midz01pre=z1pre;
								}
								if (y1pre==NaN) {
									midx01pre=x0pre;
									midy01pre=y0pre;
									midz01pre=z0pre;
								}
								dtemp=sqrt(pow(midx01-midx01pre, 2)+pow(midy01-midy01pre, 2)+pow(midz01-midz01pre, 2));
								if (roiFirst==true && f==1)dtemp=1/dtemp;
								if (precompare==false) dfin=dtemp;
								precompare=true;
							}
							
							else if (precompare==false) dtemp=sqrt(pow(x1-x0, 2)+pow(y1-y0, 2)+pow(z1-z0, 2));
							if (e==0){
								dfin=dtemp;
									if (YsPerFrame==1){
										dotposition=newArray(PAIRS[e],PAIRS[e+1],PAIRS[e+2],PAIRS[e+3],PAIRS[e+4],PAIRS[e+5]);
										XpositTemp[0]=dotposition[0];
										YpositTemp[0]=dotposition[1];
										ZpositTemp[0]=dotposition[2];
										XpositTemp[1]=dotposition[3];
										YpositTemp[1]=dotposition[4];
										ZpositTemp[1]=dotposition[5];
										}
									else{ 	
										dotposition=newArray(PAIRS[e+3],PAIRS[e+4],PAIRS[e+5]);
										XpositTemp[0]=dotposition[0]; 
										YpositTemp[0]=dotposition[1];
										ZpositTemp[0]=dotposition[2];
										}
									originA=origin[e/3];
									originB=origin[(e/3)+1];
							}
							else{
								if (dtemp<dfin){
									if (YsPerFrame==1){
										dotposition=newArray(PAIRS[e],PAIRS[e+1],PAIRS[e+2],PAIRS[e+3],PAIRS[e+4],PAIRS[e+5]);
										XpositTemp[0]=dotposition[0];
										YpositTemp[0]=dotposition[1];
										ZpositTemp[0]=dotposition[2];
										XpositTemp[1]=dotposition[3];
										YpositTemp[1]=dotposition[4];
										ZpositTemp[1]=dotposition[5];
									}
									else{ 
										dotposition=newArray(PAIRS[e+3],PAIRS[e+4],PAIRS[e+5]);
										XpositTemp[0]=dotposition[0];
										YpositTemp[0]=dotposition[1];
										ZpositTemp[0]=dotposition[2];
									}
										dfin=dtemp;
										originA=origin[e/3];
										originB=origin[(e/3)+1];
								}
							}
						}
							Xposit=Array.concat(Xposit,XpositTemp);
							Yposit=Array.concat(Yposit,YpositTemp);
							Zposit=Array.concat(Zposit,ZpositTemp);

						//Removes temporary data not used to calculate the distance between the centers of consecutive connected processes.
						coor=Array.slice(coor, originA, originB);
						for (l=0; l<coor.length; l=l+5){
							for (ll=l+5; ll<coor.length-5; ll=ll+5){
								Z=coor[l+4];
								Z2=coor[ll+4];
								if (Z==Z2){
									sliceadoL=Array.slice(coor,0, ll);
									sliceadoR=Array.slice(coor,ll+5, coor.length);
									coor=Array.concat(sliceadoL,sliceadoR);
									ll=ll-5;
								}
								if (Z==Z2-1){
									XL=coor[l];
									XR=coor[l+2];
									XL2=coor[ll];
									XR2=coor[ll+2];
									if (XL>XR2 || XR<XL2){
										sliceadoL=Array.slice(coor,0, ll);
										sliceadoR=Array.slice(coor,ll+5, coor.length);
										coor=Array.concat(sliceadoL,sliceadoR);
										ll=ll-5;
									}
								}
							}
						}
					}
					//Searching of the process is ended if a pair of Y consecutive (considering gaps) points is not detected
					else{
						coor=newArray();
						NOmaxima++;
						if (NOmaxima>jumpspace) i=height+1;
					}
				}
				//Searching of the process is ended if the distance between searching Y plains is avobe the value of the allowed gap (2 pixels by default)
				else {
						coor=newArray();
						NOmaxima++;
						if (NOmaxima>jumpspace) i=height+1;
					}
			}
			if (i<=height && coor.length>1) coorb=Array.copy(coor);
		}
		selectWindow("frame");
		close();
		if (visual==true) concatenate=concatenate+" image"+f+"=test"+f;
		
		//Generation of in-gap center coordinates
			
		for(it=1; it<Yposit.length; it++){
			if (Yposit[it-1]!=Yposit[it]-step){
				gaps=((Yposit[it]-(Yposit[it-1]))/step)-1;
				gapsX=(Xposit[it]-(Xposit[it-1]))/(gaps+1);
				gapsZ=(Zposit[it]-(Zposit[it-1]))/(gaps+1);
				gapsArrY=newArray(gaps);
				gapsArrX=newArray(gaps);
				gapsArrZ=newArray(gaps);
				for (iit=0; iit<gaps; iit++){
					gapsArrY[iit]=Yposit[it-1]+iit+step;
					gapsArrX[iit]=Xposit[it-1]+((iit+1)*gapsX);
					gapsArrZ[iit]=Zposit[it-1]+((iit+1)*gapsZ);
				}
				 gapsArrYR=Array.slice(Yposit,0,it);
				 gapsArrYL=Array.slice(Yposit,it,Yposit.length);
				 Yposit=Array.concat(gapsArrYR, gapsArrY);
				 Yposit=Array.concat(Yposit, gapsArrYL);
				 
				 gapsArrXR=Array.slice(Xposit,0,it);
				 gapsArrXL=Array.slice(Xposit,it,Xposit.length);
				 Xposit=Array.concat(gapsArrXR, gapsArrX);
				 Xposit=Array.concat(Xposit, gapsArrXL);

				 gapsArrZR=Array.slice(Zposit,0,it);
				 gapsArrZL=Array.slice(Zposit,it,Zposit.length);
				 Zposit=Array.concat(gapsArrZR, gapsArrZ);
				 Zposit=Array.concat(Zposit, gapsArrZL);
			}
		}
		
		//Process smoothing (Gaussian or Mean filtering)
		
		for(it=0; it<Xposit.length; it++){
			if (smoothtype=="Gaussian"){
				smoothedX=Xposit[it]*(range+1);
				smoothedZ=Zposit[it]*(range+1);
			}
			else {
				smoothedX=Xposit[it];
				smoothedZ=Zposit[it];
			}
			
			Gweight=range+1;
			SumGweight=range+1;
			
			for(iit=1; iit<=range; iit++){
				Gweight--;
				SumGweight=SumGweight+(Gweight*2);
				if (it-iit<0){
					LeftX=Xposit[0];
					LeftZ=Zposit[0];
				}
				else {
					LeftX=Xposit[it-iit];
					LeftZ=Zposit[it-iit];
				}
				if (it+iit>=Xposit.length){
					RightX=Xposit[Xposit.length-1];
					RightZ=Zposit[Xposit.length-1];
				}
				else{
					RightX=Xposit[it+iit];
					RightZ=Zposit[it+iit];
				}
				
				if (smoothtype=="Gaussian"){
					smoothedX=(LeftX*Gweight)+smoothedX+(RightX*Gweight);
					smoothedZ=(LeftZ*Gweight)+smoothedZ+(RightZ*Gweight);
				}
				else {
					smoothedX=LeftX+smoothedX+RightX;
					smoothedZ=LeftZ+smoothedZ+RightZ;
				}
				
			}
			if (smoothtype=="Gaussian"){
				smoothedX=smoothedX/SumGweight;
				Xposit[it]=smoothedX;
				smoothedZ=smoothedZ/SumGweight;
				Zposit[it]=smoothedZ;
			}
			else {
				smoothedX=smoothedX/((range*2)+1);
				Xposit[it]=smoothedX;
				smoothedZ=smoothedZ/((range*2)+1);
				Zposit[it]=smoothedZ;
			}	
		}
		
		//obtains the sum of distances from Xposit, Yposit and Zposit;
		sumd=0;
		for (it=0; it<(Xposit.length-1); it++){
			sumd=sumd+sqrt(pow((Xposit[it+1]-Xposit[it])*xs, 2)+pow((Yposit[it+1]-Yposit[it])*ys, 2)+pow((Zposit[it+1]-Zposit[it])*zs, 2));
			if (visual==true && f>0) {
				setBatchMode(false);
				if (it==0){
					selectWindow("test"+f);
					setSlice(Zposit[0]);
					drawLine(Xposit[0], Yposit[0], Xposit[0], Yposit[0]);
					Xcenter=Xposit[0]-X0centralPost;
					Ycenter=Yposit[0]-Y0centralPost;
					Xreal=(Xcenter*cos(0.01745329252*angle))+(Ycenter*sin(0.01745329252*angle));
					Yreal=(Ycenter*cos(0.01745329252*angle))-(Xcenter*sin(0.01745329252*angle));
					Xreal=round(Xreal+Xbounding+X0central);
					Yreal=round(Yreal+Ybounding+Y0central);
					selectWindow(name);
					setSlice(((f-1)*slices)+Zposit[0]);
					drawLine(Xreal, Yreal, Xreal, Yreal);
				}
				selectWindow("test"+f);
				setSlice(Zposit[it+1]);
				drawLine(Xposit[it+1], Yposit[it+1], Xposit[it+1], Yposit[it+1]);
				Xcenter=Xposit[it+1]-X0centralPost;
				Ycenter=Yposit[it+1]-Y0centralPost;
				Xreal=(Xcenter*cos(0.01745329252*angle))+(Ycenter*sin(0.01745329252*angle));
				Yreal=(Ycenter*cos(0.01745329252*angle))-(Xcenter*sin(0.01745329252*angle));
				Xreal=round(Xreal+Xbounding+X0central);
				Yreal=round(Yreal+Ybounding+Y0central);
				selectWindow(name);
				setSlice(((f-1)*slices)+Zposit[it+1]);
				drawLine(Xreal, Yreal, Xreal, Yreal);
			}
		}
		if (Zposit.length>0){
			Zcrop=Zposit[Zposit.length-1];
			Xcrop=Xposit[Xposit.length-1];
			Ycrop=Yposit[Yposit.length-1];
			Xcenter=Xcrop-X0centralPost;
			Ycenter=Ycrop-Y0centralPost;
			Xreal=(Xcenter*cos(0.01745329252*angle))+(Ycenter*sin(0.01745329252*angle));
			Yreal=(Ycenter*cos(0.01745329252*angle))-(Xcenter*sin(0.01745329252*angle));
			Xreal=Xreal+Xbounding+X0central;
			Yreal=Yreal+Ybounding+Y0central;
			if (f>1) {
				if (isNaN(Xcropre)==false) {
					DisTip=sqrt(pow((Xcrop-Xcropre)*xs, 2)+pow((Ycrop-Ycropre)*ys, 2)+pow((Zcrop-Zcropre)*zs, 2));
					interSpeed=DisTip/timeinterval;
					if(isNaN(DisTip)==false){
						if(isNaN(sumDisTip)){
							sumDisTip=0;
							sumInterSpeed=0;
							interVals=0;
						}
						sumDisTip=sumDisTip+DisTip;
						sumInterSpeed=sumInterSpeed+interSpeed;
						interVals++;
					}
					
				}
				else{
						DisTip=NaN;
						interSpeed=NaN;
					}
			}
			Xcropre=Xcrop;
			Ycropre=Ycrop;
			Zcropre=Zcrop;
			if (visual==true){
				selectWindow(name);
				setSlice(((f-1)*slices)+round(Zcrop));
				setColor("white");
				drawOval(round(Xreal)-2, round(Yreal)-2, 4, 4);
			}
		
			
			tablearray=newArray(fotos[fot], roinames[r], buf, f*timeinterval, Xcrop, Ycrop, Zcrop, Xreal, Yreal, Zcrop, DisTip, interSpeed);
			tableprinter("Tip velocity", tablearray);
		}
		else {
			Xcropre=NaN;
			Ycropre=NaN;
			Zcropre=NaN;
		}
		//Calculates the  motility (lenght of each process at each time frame), and the resulting net process velocity, process velocity, protraction, retraction, tip displacement, and tip velocity between consecutive frames (microns per minute)
		if (f>1){
			difference=sumd-totallength;
			motility=difference/timeinterval;
			motilitygeneral=sqrt(pow(motility, 2));
			tablearray=newArray(fotos[fot], roinames[r], buf, f*timeinterval, sumd, motility, motilitygeneral);
			tableprinter("Process velocity", tablearray);
			sumMotility=sumMotility+motility;
			if (motility>0){
				sumprotrac=sumprotrac+motility;
				protracounter++;
			}
			if (motility<0){
				sumretrac=sumretrac+motility;
				retracounter++;
			}
			sumAbsMotility=sumAbsMotility+motilitygeneral;
			contador2++;	
		}
		else{
			tablearray=newArray(fotos[fot], roinames[r], buf, f*timeinterval, sumd, NaN, NaN);
			tableprinter("Process velocity", tablearray);			
		}
		totallength=sumd;
	}
	
	/*Populates the following tables:
	Process velocity: contains the lenght of each process at each time frame, and the resulting process velocity, protraction, and retraction between consecutive frames (microns per minute)
 	Mean Process velocity: contains the mean process velocity values from each process over the time course of the experiment (microns per minute)
	 Retraction: contains the mean retraction values from each each process over the time course of the experiment (microns per minute)
	 Protraction: contains mean protraction values from each process over the time course of the experiment (microns per minute)
 	Tip velocity: contains the coordinates (XYZ) of the position of the tip in each time frame, distance travelled by the tip, and tip velocity between consecutive frames (microns per minute)
 	Mean tip velocity: contains total distance travelled by the tip, and mean tip velocity over the time course of the experiment (microns per minute)
	*/
	
	tablearray=newArray(fotos[fot], roinames[r], buf,  sumMotility/contador2, sumAbsMotility/contador2);
	tableprinter("Mean Process velocity", tablearray);

	tablearray=newArray(fotos[fot], roinames[r], buf,  sumprotrac/protracounter);
	tableprinter("Protraction", tablearray);

	tablearray=newArray(fotos[fot], roinames[r], buf,  sumretrac/retracounter);
	tableprinter("Retraction", tablearray);

	tablearray=newArray(fotos[fot], roinames[r], buf, sumDisTip, sumDisTip/(interVals*timeinterval), sumDisTip/interVals, sumInterSpeed/interVals); 
	tableprinter("Mean Tip velocity", tablearray);	
	
	
	if (visual==true){
		run("Concatenate...", ""+concatenate+"");
		selectWindow("Concatenated Stacks");
		run("Stack to Hyperstack...", "order=xyczt(default) channels=1 slices="+slices+" frames="+frames+" display=Grayscale");
		selectWindow("Concatenated Stacks");
		run("Z Project...", "projection=[Max Intensity] all");
		selectWindow("MAX_Concatenated Stacks");
		if (slices>1) run("Make Montage...", "columns=10 rows="+slices/10+" scale=1");
		selectWindow("crop");
		run("Z Project...", "projection=[Max Intensity] all");
		selectWindow(name);
		run("Z Project...", "projection=[Max Intensity] all");
		selectWindow("crop");
		close();
		selectWindow("originalcrop");
		close();
		selectWindow("MAX_Concatenated Stacks");
		close();
		selectWindow("Concatenated Stacks");
		close();
		selectWindow("MAX_crop");
		close();
		selectWindow(name);
		run("Maximize");
		if (isOpen("Montage")){
			selectWindow("Montage");
			run ("Maximize");
		}
		selectWindow("MAX_"+name);
		run("Maximize");
		run("Tile");
		waitForUser("Optimization", "Please, check the images and click OK.\n   \n                     Zoom in: Ctrl+\n                     Zoom out: Ctrl-");
		closerer();
	}
	if (isOpen("crop")){
		selectWindow("crop");
		close();
	}
	
	if (visual==true){ 
	Dialog.create("Optimization Menu");
	items = newArray("Change Parameters", "Continue", "Analyze all processes using current parameteres", "Exit");
	Dialog.addRadioButtonGroup("What would you like to do?" ,items, 4 , 1, "Continue");
	Dialog.addCheckbox("Next Cell", false);
	Dialog.addCheckbox("Next Image", false);
	Dialog.addHelp("http://www.achucarro.org/downloads");
	Dialog.show();
	Parameters = Dialog.getRadioButton();
	nextcell=Dialog.getCheckbox();
	nextimage=Dialog.getCheckbox();
	if (Parameters=="Exit" || Parameters=="Analyze all processes using current parameteres"){
				run("Close All");
				closerer();
				selectWindow("ROI Manager");
				run("Close");
				clostab=true;
				if (Parameters=="Exit") clostab=getBoolean("Do you want to close all tables?");
				else clostab=true;
				if (clostab==true){
					selectWindow("Protraction");
					run("Close");
					selectWindow("Retraction");
					run("Close");
					selectWindow("Mean Process velocity");
					run("Close");
					selectWindow("Process velocity"); 
					run("Close");
					selectWindow("Mean Tip velocity");
					run("Close");
					selectWindow("Tip velocity");
					run("Close");
			}
			saveparameters();
			if (Parameters=="Exit") exit();
			else Parameters="Analyze";
			visual=false;
		}
	}
}
		
//Functions to create, populate, collect data from, and save tables

function tablecreator(tabname, tablearray){
	run("New... ", "name=["+tabname+"] type=Table");
	headings=tablearray[0];
	for (i=1; i<tablearray.length; i++) headings=headings+"\t"+tablearray[i];
	print("["+tabname+"]", "\\Headings:"+ headings);	
}

function tableprinter(tabname, tablearray){
	line=toString(tablearray[0]);
	for (i=1; i<tablearray.length; i++) line=line+"\t"+tablearray[i];
	print ("["+tabname+"]", line);
}




function infoTab(tablename, line, column){
	selectWindow(tablename);
	tableinfo=getInfo();
	Ltab=split(tableinfo, "\n");
	Ctab=split(Ltab[line], "\t");
	infovar=Ctab[column];
}


function  savetab(tablename, path){
		selectWindow(tablename);
		saveAs("Text", path+tablename+".xls");
}

	
	
	
		
function frameprojector(projectype, imageout){
	selectWindow(imageout);
	getDimensions(width, height, channels, slices, frames);
	for (i=2; i<frames; i++){
		if (i==2){
			if (slices>1) run("Duplicate...", "title=transformed duplicate frames=1");
			else {
				setSlice(1);
		 		run("Duplicate...", "title=transformed");
			}
		}
		selectWindow(imageout);
		if (slices>1) run("Duplicate...", "title="+imageout+"-"+i+" duplicate frames="+1+i);
		else {
			setSlice(1+i);
		 	run("Duplicate...", "title="+imageout+"-"+i);
		}
		imageCalculator(projectype+" stack", "transformed",imageout+"-"+i);
		selectWindow(imageout+"-"+i);
		close();
	}
	selectWindow(imageout);
	close();
	selectWindow("transformed");
	rename(imageout);
}	
		
		
//Function that performs the pre-detection step which improves the quality of the reconstruction and serves as a quality check	
function Prefilamentdetector(morethanone){
	//Obtains the dimensions of the image and creates a series of variables used to analyze the motility
	setBatchMode(true);
	getDimensions(width, height, channels, slices, frames);
	contador2=0;
	sumMotility=0;
	sumAbsMotility=0;
	sumretrac=0;
	sumprotrac=0;
	retracounter=0;
	sumprotrac=0;
	protracounter=0;
	YpositTOTAL=newArray();
	XpositTOTAL=newArray();
	ZpositTOTAL=newArray();
	YpositAVO=newArray();
	XpositAVO=newArray();
	ZpositAVO=newArray();
	//This loop determines the process lenght in each consecutive time frame
	for (f=1; f<=frames; f++){
		NOmaxima=0;
		YsPerFrame=0; 
		if (f==1 && avoid==true){
			selectWindow("MeanZYZpoints");
			tableinfo=getInfo();
			Ltab=split(tableinfo, "\n");
			sizearr=Ltab.length;
			XpositPrev=newArray(sizearr);
			YpositPrev=newArray(sizearr);
			ZpositPrev=newArray(sizearr);
			for (i=1; i<Ltab.length; i++){
				infoTab("MeanZYZpoints", i, 0);
				YpositPrev[i-1]=parseFloat(infovar);
				infoTab("MeanZYZpoints", i, 1);
				XpositPrev[i-1]=parseFloat(infovar);
				infoTab("MeanZYZpoints", i, 2);
				ZpositPrev[i-1]=parseFloat(infovar);	
			}
			selectWindow("MeanZYZpoints");
			//run("Close");
		}
		if (f>1){
			XpositPrev=Array.copy(Xposit);
			YpositPrev=Array.copy(Yposit);
			ZpositPrev=Array.copy(Zposit);	
		}
		Xposit=newArray();
		Yposit=newArray();
		Zposit=newArray();
	 	selectWindow("crop");
	 	run("Select None");
	 	if (slices>1) run("Duplicate...", "title=frame duplicate frames="+f);
	 	else {
	 		setSlice(f);
	 		run("Duplicate...", "title=frame");
	 	}
		selectWindow("frame");
		sumd=0;
		coorb=newArray();
		//This loop draws profile lines through the Y axis of the cropped image at the selected Y steps ("step")
		for (i=0; i<=height; i=i+step){
			coor=newArray();
			coortemp=newArray(5);
			
			//This loop will draw the profile lines at each Y step through the Z axis
			for (stack=1; stack<=slices; stack++){
				fibra=0;
				selectWindow("frame");
				//Obtains the intensity data of each profile line and creates an array with the XYZ position of the intensity peaks
				// using the selected maximum intensity difference between peak and valley ("maxtolerance" variable)
				setSlice(stack);
				makeLine(0, i, width, i);
				prof=getProfile();
				max=Array.findMaxima(prof, maxtolerance);
				Array.sort(max);
				//This loop will determine the borders of the process at each Y position
				for (ii=0; ii<max.length; ii++){
					//Determines the X coordinates of the right border as a ratio ("ratio") of the maximum peak intensity ("minInt")
					//The position of the peak ("origin") is the starting point for searching the borders.
					counterR=0;
					counter2R=0;
					origin=max[ii];
					//Borders will be determined only if the maxima intensity is above the threshold ("thrsval")
					if (prof[origin]>=thrsval){
						dif=0;
						if (absolutborder==true) minInt=prof[origin]*ratio;
						else minInt=thrsval;
						if (origin+1<prof.length){
							//Loop to compare intensities through the profile line between them and with the intensity to define a border ("minInt")
							//In case the process splits into two terminal branches, the defined border inflexion amplitude
							//("maxdif") will be used to determine the border as the valley between two peaks
							out=0;
							do{
								counter2R++;
								dif=prof[origin+counterR]-prof[origin+counter2R];
								if (prof[origin+counter2R]<minInt) out=1;
								else if(prof[origin+counter2R]<prof[origin+counterR]) counterR=counter2R;
							}while ((origin+counter2R+1)<prof.length && dif>maxdif && out==0);
							if (dif>maxdif) counter2R=counterR;
						}
						
						//Determines the X coordinates of the left border as a ratio ("ratio") of the maximum peak intensity ("minInt")
						//The position of the peak ("origin") is the starting point for searching the borders.
						counterL=0;
						counter2L=0;
						dif=0;
						if (origin-1>=0){
							//Loop to compare intensities through the profile line between them and with the intensity to define a border ("minInt")
							//In case the process splits into two terminal branches, the defined border inflexion amplitude
							//("maxdif") will be used to determine the border as the valley between two peaks
							out=0;
							do{
								counter2L++;
								dif=prof[origin-counterL]-prof[origin-counter2L];
								if (prof[origin-counter2L]<minInt) out=1;
								else if (prof[origin-counter2L]<prof[origin-counterL]) counterL=counter2L;	
							} while ((origin-counter2L-1)>=0 && dif>maxdif && out==0);
							if (dif>maxdif) counter2L=counterL;
						}
						
						//X coordinates of the left ("positionL") and right ("positionR") borders are used to obtain the X coordinate of the center of the process ("position")
						//The (5) coordinates of the left border (X), center (X), right border (X) at each Y and Z coordinate are stored in the array "coor" 
						positionL=origin-counter2L;
						positionR=origin+counter2R;
						position=((positionR+positionL)/2);
						coortemp[0]=positionL;
						coortemp[1]=position;
						coortemp[2]=positionR;
						coortemp[3]=i;
						coortemp[4]=stack;
						coor=Array.concat(coor, coortemp);
						fibra++;
					}
				}
			}
			//If more than two borders are found at the same YZ location, it discriminates whether there is a single or more processes
			if(coor.length>5){
				counter=0;
				//Loop to merge connected processes (in X). Each group of 5 coordinates is compared with the others so that if an overlap between the line defined by the X coordinates at each Y and Z position, 
				//the two most distal X coordinates are taken as the borders of a single process.
				for (arele=0; arele<coor.length-5; arele=arele+5) {
					counter=arele+5;
					for(arele2=counter; arele2<coor.length; arele2=arele2+5){	
						Z=coor[arele+4];
						Z2=coor[arele2+4];
						if (Z==Z2){
							XL=coor[arele];
							XR=coor[arele+2];
							XL2=coor[arele2];
							XR2=coor[arele2+2];
							if(XL<=XR2 && XR>=XL2) {
								if (XL<XL2) coor[arele]=XL;
								else coor[arele]=XL2;
								if (XR>XR2) coor[arele+2]=XR;
								else coor[arele+2]=XR2;
								coor[arele+1]=(coor[arele]+coor[arele+2])/2;
								coorL=Array.slice(coor,0,arele2);
								coorR=Array.slice(coor, arele2+5, coor.length); 
								coor=Array.concat(coorL, coorR);
								arele2=arele2-5;
								counter=counter-5;
							}
						}
					}
				}
			}
			
			//Loop to obtain the distance between the process center at each defined Y position ("step"). 
						
			if(coorb.length>1 && coor.length>1){
				NOmaxima=(coor[3]-coorb[3]-step);
				if((coorb[3]==(coor[3]-step)-NOmaxima) && NOmaxima<=jumpspace){
					
					Zgapranger=NOmaxima*zgapper;
					Xgapranger=NOmaxima*xgapper;
					NOmaxima=0;
					PAIRS=newArray();
					origin=newArray();
					origintemp=newArray(2);

					//Loop to identify connected processes (in Y). Each group of X, Z coordinates is compared in consecutive Y positions so that if an overlap of the area defined by X,Z is found, the center of this area is determined (array "PAIRS").
					//The distance between the center of each pair of connected areas is calculated. Only the minimal distance between consecutive Y positions is stored to calculate the total length.
					//If a second terminal branch appears, its distance to the previous center will be larger and not taken into account. Thus, only the length of one terminal branch is calculated.
					for (w=0;w<coorb.length;w=w+5){
						for(ww=0;ww<coor.length;ww=ww+5){
							Z=coorb[w+4];
							Z2=coor[ww+4];
							if (Z>=(Z2-Zgapranger) && Z<=(Z2+Zgapranger)){
								XL=coorb[w];
								XR=coorb[w+2];
								XL2=coor[ww];
								XR2=coor[ww+2];
								if (XL<=(XR2+Xgapranger) && XR>=(XL2-Xgapranger)){
									origintemp[0]=ww;
									origintemp[1]=ww+5;
									
									//Loop to obtain the deep X, Z coordinates to define the area occupied by the process in each Y position
									RA=newArray();
									poscomp=w;
									for(arele=w+5; arele<coorb.length; arele=arele+5){
										Z=coorb[poscomp+4];
										Z2=coorb[arele+4];
										if (Z==Z2-1){
											XL=coorb[poscomp];
											XR=coorb[poscomp+2];
											XL2=coorb[arele];
											XR2=coorb[arele+2];
											if (XL<=XR2 && XR>=XL2){
												RAT=newArray(XL2, XR2, coorb[arele+3], coorb[arele+4]);
												RA=Array.concat(RA,RAT);
												poscomp=arele;
											}
										}
									}
											
									//Loop to obtain the surface X, Z coordinates to define the area occupied by the process in each Y position
									LA=newArray();
									poscomp=w;									
									for(arele=w-5; arele>=0; arele=arele-5){
										Z=coorb[poscomp+4];
										Z2=coorb[arele+4];
										if (Z==Z2+1){
											XL=coorb[poscomp];
											XR=coorb[poscomp+2];
											XL2=coorb[arele];
											XR2=coorb[arele+2];
											if (XL<=XR2 && XR>=XL2){
												LAT=newArray(XL2, XR2, coorb[arele+3], coorb[arele+4]);
												LA=Array.concat(LAT,LA);
												poscomp=arele;
											}
										}
									}
											
									//Creates an array ("finalY") that defines the X, Z area occupied by the process at each Y position
									MA=newArray(coorb[w], coorb[w+2], coorb[w+3], coorb[w+4]);
									finalY=Array.concat(LA,MA);
									finalY=Array.concat(finalY, RA);
																									
									//Calculates the center point of the process in the Z axis 
									contador=0;
									sumatorio=0;
									for (psp=3; psp<finalY.length; psp=psp+4){
										contador++;
										sumatorio=sumatorio+finalY[psp];
									}
									Zmed=sumatorio/contador;

									//Calculates the center point of the process in the X axis 							
									ZmedEnt=round(Zmed);
									POSYtemp=newArray(3);
									if (Zmed!=ZmedEnt){
										for (psp=3; psp<finalY.length; psp=psp+4){
											contador++;
											if (finalY[psp]==ZmedEnt){
														
												POSYtemp[0]=(finalY[psp-3]+finalY[psp-2]+finalY[psp-7]+finalY[psp-6])/4;
												POSYtemp[1]=finalY[psp-1];
												POSYtemp[2]=Zmed;
											}
										}
									}
									else {
										for (psp=3; psp<finalY.length; psp=psp+4){
											contador++;
											if (finalY[psp]==Zmed){
														
												POSYtemp[0]=(finalY[psp-3]+finalY[psp-2])/2;
												POSYtemp[1]=finalY[psp-1];
												POSYtemp[2]=Zmed;
											}
										} 
									}
									
									//The X, Z coordinates of the center of the process at the current Y position are stored in "PAIRS"
									PAIRS=Array.concat(PAIRS, POSYtemp);

									//The same procedure is repeated for the next Y position, as defined by "step"
									
									//Loop to obtain the deep X, Z coordinates to define the area occupied by the process in each Y position
									RA=newArray();
									poscomp=ww;
									for(arele=ww+5; arele<coor.length; arele=arele+5){
											
										Z=coor[poscomp+4];
										Z2=coor[arele+4];
				
										if (Z==Z2-1){
											XL=coor[poscomp];
											XR=coor[poscomp+2];
											XL2=coor[arele];
											XR2=coor[arele+2];
											if (XL<=XR2 && XR>=XL2){
												RAT=newArray(XL2, XR2, coor[arele+3], coor[arele+4]);
												RA=Array.concat(RA,RAT);
												origintemp[1]=arele+5;
												poscomp=arele;
											}
										}
									}
											
									//Loop to obtain the surface X, Z coordinates to define the area occupied by the process in each Y position
									LA=newArray();
									poscomp=ww;
										
									for(arele=ww-5; arele>=0; arele=arele-5){
											
										Z=coor[poscomp+4];
										Z2=coor[arele+4];
										if (Z==Z2+1){
											XL=coor[poscomp];
											XR=coor[poscomp+2];
											XL2=coor[arele];
											XR2=coor[arele+2];
											if (XL<=XR2 && XR>=XL2){
												LAT=newArray(XL2, XR2, coor[arele+3], coor[arele+4]);
												LA=Array.concat(LAT,LA);
												origintemp[0]=arele;
												poscomp=arele;
											}
										}
									}
									origin=Array.concat(origin, origintemp);
								
								    //Creates an array ("finalY") that defines the X, Z area occupied by the process at each Y position
									MA=newArray(coor[ww], coor[ww+2], coor[ww+3], coor[ww+4]);
									finalY=Array.concat(LA,MA);
									finalY=Array.concat(finalY, RA);
									
									//Calculates the center point of the process in the Z axis 
									contador=0;
									sumatorio=0;
									for (psp=3; psp<finalY.length; psp=psp+4){
										contador++;
										sumatorio=sumatorio+finalY[psp];
									}
									Zmed=sumatorio/contador;
									
									//Calculates the center point of the process in the X axis 	
									ZmedEnt=round(Zmed);
									POSY2temp=newArray(3);
									if (Zmed!=ZmedEnt){
										for (psp=3; psp<finalY.length; psp=psp+4){
											contador++;
											if (finalY[psp]==ZmedEnt){
														
												POSYtemp[0]=(finalY[psp-3]+finalY[psp-2]+finalY[psp-7]+finalY[psp-6])/4;
												POSYtemp[1]=finalY[psp-1];
												POSYtemp[2]=Zmed;
											}
										}
									}
									else {
										for (psp=3; psp<finalY.length; psp=psp+4){
											contador++;
											if (finalY[psp]==Zmed){
														
												POSYtemp[0]=(finalY[psp-3]+finalY[psp-2])/2;
												POSYtemp[1]=finalY[psp-1];
												POSYtemp[2]=Zmed;
											}
										} 
									}
									//The X, Z coordinates of the center of the process at the current Y position are stored in "PAIRS";
									PAIRS=Array.concat(PAIRS, POSYtemp);
								}
							}
						}
					}
					
		
					//Removes potential repeated pairs of coordinates of the process center
					origintemp=newArray(2);
					for (w=0; w<PAIRS.length-6; w=w+6){
						XA=PAIRS[w];
						YA=PAIRS[w+1];
						ZA=PAIRS[w+2];
						XAA=PAIRS[w+3];
						YAA=PAIRS[w+4];
						ZAA=PAIRS[w+5];
						for (ww=6; ww<PAIRS.length; ww=ww+6){
							XB=PAIRS[ww];
							YB=PAIRS[ww+1];
							ZB=PAIRS[ww+2];
							XBB=PAIRS[ww+3];
							YBB=PAIRS[ww+4];
							ZBB=PAIRS[ww+5];
							if (XA==XB && YA==YB && ZA==ZB && XAA==XBB && YAA==YBB && ZAA==ZBB && w!=ww){
								PAIRSL=Array.slice(PAIRS,0,ww);
								PAIRSR=Array.slice(PAIRS, ww+6, PAIRS.length); 
								PAIRS=Array.concat(PAIRSL, PAIRSR);
								origintemp[0]=origin[w/3];
								origintemp[1]=origin[(ww/3)+1];
								origin[w/3]=origintemp[0];
								origin[w/3+1]=origintemp[1];
								originL=Array.slice(origin,0,ww/3);
								originR=Array.slice(origin, (ww/3)+2, origin.length);
								origin=Array.concat(originL, originR);
								ww=ww-6;
							}
						}
					}
					
					if (PAIRS.length>0){
						//Transforms pixels into length (as defined by the image file parameters) and caldulates the distance between the centers of consecutive connected processes. Only the minimal distance is stored.
						//The total length is calculated as the sum of the mimimal distances
						YsPerFrame++;
						if (YsPerFrame==1){
								XpositTemp=newArray(2);
								YpositTemp=newArray(2);
								ZpositTemp=newArray(2);
							}
						else{ 
								XpositTemp=newArray(1);
								YpositTemp=newArray(1);
								ZpositTemp=newArray(1);
							}
							precompare=false;
						for (e=0; e<PAIRS.length; e=e+6){
							
							getVoxelSize(xs, ys, zs, unit);
							x1=PAIRS[e+3]*xs;
							x0=PAIRS[e]*xs;
							y1=PAIRS[e+4]*ys;
							y0=PAIRS[e+1]*ys;
							z1=PAIRS[e+5]*zs;
							z0=PAIRS[e+2]*zs;
							
							midx01=(x1+x0)/2;
							midy01=(y1+y0)/2;
							midz01=(z1+z0)/2;
							//Distances of the current pairs of points to the previous process in the same Ys
							
							y0pre=NaN;
							y1pre=NaN;
							x0pre=NaN;
							x1pre=NaN;
							z0pre=NaN;
							z1pre=NaN;
							recounter=0;
							
							if (f>1 || (avoid==true && f==1)){
								do {
									for (dd=0; dd<YpositPrev.length; dd++){
										if (YpositPrev[dd]==PAIRS[e+1]+recounter || YpositPrev[dd]==PAIRS[e+1]-recounter ){
											x0pre=XpositPrev[dd]*xs;
											z0pre=ZpositPrev[dd]*zs;
											y0pre=YpositPrev[dd]*ys;
											nnn=1;
											for (hh=dd+1; hh<YpositPrev.length && hh<dd+AverPre; hh++){
												x0pre=x0pre+XpositPrev[hh]*xs;
												z0pre=z0pre+ZpositPrev[hh]*zs;
												y0pre=y0pre+YpositPrev[hh]*ys;
												nnn++;
											}
											x0pre=x0pre/nnn;
											z0pre=z0pre/nnn;
											y0pre=y0pre/nnn;
										}
										if (YpositPrev.length>dd+1){
											if ((YpositPrev[dd+1]==PAIRS[e+4]+recounter) || (YpositPrev[dd+1]==PAIRS[e+4]-recounter)){
												x1pre=XpositPrev[dd+1]*xs;
												z1pre=ZpositPrev[dd+1]*zs;
												y1pre=YpositPrev[dd+1]*ys;
												nnn=1;
												for (hh=dd+2; hh<YpositPrev.length && hh<dd+AverPre; hh++){
													x1pre=x1pre+XpositPrev[hh]*xs;
													z1pre=z1pre+ZpositPrev[hh]*zs;
													y1pre=y1pre+YpositPrev[hh]*ys;
													nnn++;
												}
												x1pre=x1pre/nnn;
												z1pre=z1pre/nnn;
												y1pre=y1pre/nnn;
											}	
										}
									}
									recounter++; 
								}
								while((y0pre==NaN && y1pre==NaN) && recounter<=stepper);
							}
					
						
							if (y0pre!=NaN || y1pre!=NaN){
								if (y0pre!=NaN && y1pre!=NaN){
									midx01pre=(x1pre+x0pre)/2;
									midy01pre=(y1pre+y0pre)/2;
									midz01pre=(z1pre+z0pre)/2;
								}
								if (y0pre==NaN) {
									midx01pre=x1pre;
									midy01pre=y1pre;
									midz01pre=z1pre;
								}
								if (y1pre==NaN) {
									midx01pre=x0pre;
									midy01pre=y0pre;
									midz01pre=z0pre;
								}
								dtemp=sqrt(pow(midx01-midx01pre, 2)+pow(midy01-midy01pre, 2)+pow(midz01-midz01pre, 2));
								if (avoid==true && f==1)dtemp=1/dtemp;

								if (precompare==false) dfin=dtemp;
								precompare=true;
							}
							
							else if (precompare==false) dtemp=sqrt(pow(x1-x0, 2)+pow(y1-y0, 2)+pow(z1-z0, 2));
							if (e==0){
								dfin=dtemp;
									if (YsPerFrame==1){
										dotposition=newArray(PAIRS[e],PAIRS[e+1],PAIRS[e+2],PAIRS[e+3],PAIRS[e+4],PAIRS[e+5]);
										XpositTemp[0]=dotposition[0];
										YpositTemp[0]=dotposition[1];
										ZpositTemp[0]=dotposition[2];
										XpositTemp[1]=dotposition[3];
										YpositTemp[1]=dotposition[4];
										ZpositTemp[1]=dotposition[5];
										}
									else{ 	
										dotposition=newArray(PAIRS[e+3],PAIRS[e+4],PAIRS[e+5]);
										XpositTemp[0]=dotposition[0]; 
										YpositTemp[0]=dotposition[1];
										ZpositTemp[0]=dotposition[2];
										}
									originA=origin[e/3];
									originB=origin[(e/3)+1];
							}
							else{
								if (dtemp<dfin){
									if (YsPerFrame==1){
										dotposition=newArray(PAIRS[e],PAIRS[e+1],PAIRS[e+2],PAIRS[e+3],PAIRS[e+4],PAIRS[e+5]);
										XpositTemp[0]=dotposition[0];
										YpositTemp[0]=dotposition[1];
										ZpositTemp[0]=dotposition[2];
										XpositTemp[1]=dotposition[3];
										YpositTemp[1]=dotposition[4];
										ZpositTemp[1]=dotposition[5];
									}
									else{ 
										dotposition=newArray(PAIRS[e+3],PAIRS[e+4],PAIRS[e+5]);
										XpositTemp[0]=dotposition[0];
										YpositTemp[0]=dotposition[1];
										ZpositTemp[0]=dotposition[2];
									}
										dfin=dtemp;
										originA=origin[e/3];
										originB=origin[(e/3)+1];
								}
							}
						}
							Xposit=Array.concat(Xposit,XpositTemp);
							Yposit=Array.concat(Yposit,YpositTemp);
							Zposit=Array.concat(Zposit,ZpositTemp);

						//Removes temporary data not used to calculate the distance between the centers of consecutive connected processes.
						coor=Array.slice(coor, originA, originB);
						for (l=0; l<coor.length; l=l+5){
							for (ll=l+5; ll<coor.length-5; ll=ll+5){
								Z=coor[l+4];
								Z2=coor[ll+4];
								if (Z==Z2){
									sliceadoL=Array.slice(coor,0, ll);
									sliceadoR=Array.slice(coor,ll+5, coor.length);
									coor=Array.concat(sliceadoL,sliceadoR);
									ll=ll-5;
								}
								if (Z==Z2-1){
									XL=coor[l];
									XR=coor[l+2];
									XL2=coor[ll];
									XR2=coor[ll+2];
									if (XL>XR2 || XR<XL2){
										sliceadoL=Array.slice(coor,0, ll);
										sliceadoR=Array.slice(coor,ll+5, coor.length);
										coor=Array.concat(sliceadoL,sliceadoR);
										ll=ll-5;
									}
								}
							}
						}
					}
					//Searching of the process is ended if a pair of Y consecutive (considering gaps) points is not detected
					else{
						coor=newArray();
						NOmaxima++;
						if (NOmaxima>jumpspace) i=height+1;
					}
				}
				//Searching of the process is ended if the distance between searching Y plains is avobe the value of the allowed gap (2 pixels by default)
				else {
						coor=newArray();
						NOmaxima++;
						if (NOmaxima>jumpspace) i=height+1;
					}
			}
			if (i<=height && coor.length>1) coorb=Array.copy(coor);
		}
		selectWindow("frame");
		close();
		
		//Generation of in-gap center coordinates
			
		for(it=1; it<Yposit.length; it++){
			if (Yposit[it-1]!=Yposit[it]-step){
				gaps=((Yposit[it]-(Yposit[it-1]))/step)-1;
				gapsX=(Xposit[it]-(Xposit[it-1]))/(gaps+1);
				gapsZ=(Zposit[it]-(Zposit[it-1]))/(gaps+1);
				gapsArrY=newArray(gaps);
				gapsArrX=newArray(gaps);
				gapsArrZ=newArray(gaps);
				for (iit=0; iit<gaps; iit++){
					gapsArrY[iit]=Yposit[it-1]+iit+step;
					gapsArrX[iit]=Xposit[it-1]+((iit+1)*gapsX);
					gapsArrZ[iit]=Zposit[it-1]+((iit+1)*gapsZ);
				}
				 gapsArrYR=Array.slice(Yposit,0,it);
				 gapsArrYL=Array.slice(Yposit,it,Yposit.length);
				 Yposit=Array.concat(gapsArrYR, gapsArrY);
				 Yposit=Array.concat(Yposit, gapsArrYL);
				 
				 gapsArrXR=Array.slice(Xposit,0,it);
				 gapsArrXL=Array.slice(Xposit,it,Xposit.length);
				 Xposit=Array.concat(gapsArrXR, gapsArrX);
				 Xposit=Array.concat(Xposit, gapsArrXL);

				 gapsArrZR=Array.slice(Zposit,0,it);
				 gapsArrZL=Array.slice(Zposit,it,Zposit.length);
				 Zposit=Array.concat(gapsArrZR, gapsArrZ);
				 Zposit=Array.concat(Zposit, gapsArrZL);
			}
		}
		
		//Process smoothing (Gaussian or Mean filtering)
		
		for(it=0; it<Xposit.length; it++){
			if (smoothtype=="Gaussian"){
				smoothedX=Xposit[it]*(range+1);
				smoothedZ=Zposit[it]*(range+1);
			}
			else {
				smoothedX=Xposit[it];
				smoothedZ=Zposit[it];
			}
			
			Gweight=range+1;
			SumGweight=range+1;
			
			for(iit=1; iit<=range; iit++){
				Gweight--;
				SumGweight=SumGweight+(Gweight*2);
				if (it-iit<0){
					LeftX=Xposit[0];
					LeftZ=Zposit[0];
				}
				else {
					LeftX=Xposit[it-iit];
					LeftZ=Zposit[it-iit];
				}
				if (it+iit>=Xposit.length){
					RightX=Xposit[Xposit.length-1];
					RightZ=Zposit[Xposit.length-1];
				}
				else{
					RightX=Xposit[it+iit];
					RightZ=Zposit[it+iit];
				}
				
				if (smoothtype=="Gaussian"){
					smoothedX=(LeftX*Gweight)+smoothedX+(RightX*Gweight);
					smoothedZ=(LeftZ*Gweight)+smoothedZ+(RightZ*Gweight);
				}
				else {
					smoothedX=LeftX+smoothedX+RightX;
					smoothedZ=LeftZ+smoothedZ+RightZ;
				}
				
			}
			if (smoothtype=="Gaussian"){
				smoothedX=smoothedX/SumGweight;
				Xposit[it]=smoothedX;
				smoothedZ=smoothedZ/SumGweight;
				Zposit[it]=smoothedZ;
			}
			else {
				smoothedX=smoothedX/((range*2)+1);
				Xposit[it]=smoothedX;
				smoothedZ=smoothedZ/((range*2)+1);
				Zposit[it]=smoothedZ;
			}	
		}
			//Selects skeletons that reach a determined height (height comparison)
			
			if (Yposit.length>=(perFiber*height)/100){
				YpositTOTAL=Array.concat(YpositTOTAL, Yposit);
				XpositTOTAL=Array.concat(XpositTOTAL, Xposit);
				ZpositTOTAL=Array.concat(ZpositTOTAL, Zposit);
				avoid=false;
			}
			else{ 
				YpositAVO=Array.concat(YpositAVO, Yposit);
				XpositAVO=Array.concat(XpositAVO, Xposit);
				ZpositAVO=Array.concat(ZpositAVO, Zposit);
			}
	}
	 //Skeletons that do not reach a certain percentage of the selection height are excluded from the analysis 
	 if (YpositTOTAL.length<1){
	 	YpositTOTAL=Array.copy(YpositAVO);
	 	XpositTOTAL=Array.copy(XpositAVO);
	 	ZpositTOTAL=Array.copy(ZpositAVO);
	 	if (avoid==false) avoid=true;
	 	else exclude=true;
	 }
	if (exclude==false){
		mediarray();
		if (avoid==true) Prefilamentdetector(1);
	}
	if (exclude==false && morethanone==0) filamentdetector();
}
			
//Function that creates a virtual skeleton using the mean value of all the skeletons selected in the pre-detection step
function mediarray(){
	if (isOpen("MeanZYZpoints")) {
		selectWindow("MeanZYZpoints");
		run("Close");
	}
	tablearray=newArray("Y", "X", "Z");
	tablecreator("MeanZYZpoints", tablearray);
	repeated=newArray(0);
	for (i=0; i<YpositTOTAL.length; i++){
		sumvalX=0;
		sumvalZ=0;
		n=0;
		preused=0;
		for (ii=0; ii<repeated.length; ii++){
			if (YpositTOTAL[i]==repeated[ii]){
				preused=1;
				ii=repeated.length;
			}
		}
		if (preused==0){
			repeated=Array.concat(repeated,YpositTOTAL[i]);
			sumvalX=XpositTOTAL[i];
			sumvalZ=ZpositTOTAL[i];
			n=1;
			for (iii=i+1; iii<YpositTOTAL.length; iii++){
				if (YpositTOTAL[i]==YpositTOTAL[iii]){
						sumvalX=sumvalX+XpositTOTAL[iii];
						sumvalZ=sumvalZ+ZpositTOTAL[iii];
						n++;
				}	
			}
			meanvalX=sumvalX/n;
			meanvalZ=sumvalZ/n;	
			tablearray=newArray(YpositTOTAL[i], meanvalX, meanvalZ); 
			tableprinter("MeanZYZpoints", tablearray);
		}
	}
}

//Function that closes windows
function closerer(){
	if (isOpen("MAX_crop")) {
			selectWindow("MAX_crop");
			close();
		}
		
		if (isOpen("Concatenated Stacks")){
			selectWindow("Concatenated Stacks");
			close();
		}
		
		if (isOpen("MAX_Concatenated Stacks")){
			selectWindow("MAX_Concatenated Stacks");
			close();
		}

		if (isOpen("Montage")){
			selectWindow("Montage");
			close();
		}
		if (isOpen("originalcrop")){
			selectWindow("originalcrop");
			close();
		}
		if (isOpen("MAX_"+name)){
			selectWindow("MAX_"+name);
			close();
		}
}
